package sound;

import  java.io.InputStream;
import  java.io.IOException;
import  cern.jet.random.Normal;
import  cern.jet.random.engine.DRand;

import  java.io.FileOutputStream;
import  java.io.ByteArrayInputStream;
import  java.io.ByteArrayOutputStream;

import  utilities.WavRoutines;
import  wavelet.WindowedSegmentation;
import  sound.Segment;
import  sound.AudioSample;

import  javax.sound.sampled.AudioFormat;
import  javax.sound.sampled.AudioFileFormat;
import  javax.sound.sampled.AudioInputStream;
import  javax.sound.sampled.AudioSystem;
import  javax.sound.sampled.LineUnavailableException;
import  javax.sound.sampled.LineListener;
import  javax.sound.sampled.LineEvent;
import  javax.sound.sampled.Mixer;
import  javax.sound.sampled.SourceDataLine;
import  javax.sound.sampled.DataLine;
import  javax.sound.sampled.FloatControl;

/**
 * class for finally playing the segmented sound
 */
public class ByteChannel
    implements      LineListener, Runnable
{
    private Thread soundThread;
    private static final boolean    DEBUG = false;
    public SourceDataLine           m_line;
    
    /**
     * start and endpoints of segments in bytes 
     * defined in WindowSegmentation
     **/    
    public int[][]                 m_transList;
    
    /**
     * sound plays when this is true
     */
    public  boolean                 keepGoing = false;
    private int                     bytesPerWindow;
    
    /** 
     * audio associated with this class.   
     */
    AudioSample audioSample;
    WindowedSegmentation   winSeg;

    // file stuff 
    boolean                outToFile            = false; 
    FileOutputStream       outputFile;
    ByteArrayOutputStream  byteArray;
    Segment []    segment;

    // true means we need to update our list of what segments
    // go best after what. 
    boolean invNeedsUpdate;
    double  noise = 0.2;
    
    public float xFadeWins;
           int xFadeBytes;
    
    double[] mixIn1;
    double[] mixIn2;
    double[] mixOut;
    byte  [] bMixOut;

    float  chunkTimeSoFar;
    float  bytesPerSecond;

    double chunkTime;   // in seconds
    double chunkVar;    // in seconds
    double period = 0;
    double periodVar = 0;

    Normal randPeriod;
    Normal randChunk;
    DRand randEl;

    float gainS= 0.0F;
    float gainM= 0.0F;
    float gainE= 0.0F;
    float panS = 0.0F;
    float panM = 0.0F;
    float panE = 0.0F;

    MyShift panShift;
    MyShift gainShift;

    boolean doWalk = false;
    RandomWalk randWalk;

    boolean nowPlaying = false;

    /// run stuff
    int      rand;
    int      nBytesWritten;
    int      firstWin = 0; int lastWin = 0;
    double[] inter; 

    /**
     * sets stero balance
     */
    public void setPan(float m1, float m2, float m3) {
	// System.out.println("byteChannel: set pan" + m1 + " to " +  m2);
	panS = (m1*2)-1;
	panM = (m2*2)-1;
	panE = (m3*2)-1;
    }

    /**
     * @param m1 start --  takes float 0-1
     * @param m2 middle -- takes float 0-1
     * @param m3 end --    takes float 0-1
     */
    public void setGain(float m1, float m2, float m3) {
	// System.out.println("byteChannel: set pan" + m1 + " to " +  m2);
	float offset = 0;

	if (gainShift != null) {	    
	    gainS = 
		(m1*(gainShift.getMaximum()-gainShift.getMinimum()-offset)) +
		gainShift.getMinimum() + offset;
	    gainM = 
		(m2*(gainShift.getMaximum()-gainShift.getMinimum()-offset)) +
		gainShift.getMinimum() + offset;
	    gainE = 
		(m3*(gainShift.getMaximum()-gainShift.getMinimum() -offset)) +
		gainShift.getMinimum() + offset;
	}
	else {
	    gainS = (m1*80) - 66.1F;
	    gainM = (m2*80) - 66.1F;
	    gainE = (m3*80) - 66.1F;
	}
    }

    /**
     * initializes class
     * @param winSeg associated class to perform signal segmentation
     * @param aus original segmented audioSample 
     * @param transList list of transitions over sound sample
     * @param xFadeWin number of windows to crossfade over
     */
    public void init(WindowedSegmentation winSeg, AudioSample aus,
		     float xFadeWin) 
    {
	Mixer mixer    = AudioSystem.getMixer(null);	    
	audioSample    = aus;
	this.winSeg    = winSeg;
	m_transList    = winSeg.createTransList();
	invNeedsUpdate = true;
	
	xFadeWins = xFadeWin;
	bytesPerWindow  = audioSample.getSizeInBytes()*winSeg.windowSize;
	bytesPerSecond  = audioSample.getBytesPerSecond();

	randEl = new DRand(new java.util.Date());
	// initialize periodVar, chunkVar first! Where do I get this data? 
	randPeriod = new Normal((double) period, (double) periodVar, randEl);
	randChunk =  new Normal((double) chunkTime, (double) chunkVar, randEl);

	xFadeBytes      = (int)(xFadeWins*bytesPerWindow);
	try {
	    DataLine.Info   info = new DataLine.Info(SourceDataLine.class,
						     audioSample.format);
	    m_line = (SourceDataLine) mixer.getLine(info);
	    m_line.open(audioSample.format, m_line.getBufferSize());
	    
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
	
	panShift = new MyShift(m_line, FloatControl.Type.PAN);
	gainShift = new MyShift(m_line, FloatControl.Type.MASTER_GAIN); 

	// want length of window in bytes. 
	if (DEBUG) {
	    System.out.println("xFadeBytes: " + xFadeBytes);
	    System.out.println("xFadeWins: " + xFadeWins);
	}
	mixOut =  new double[(int)((winSeg.windowSize)*xFadeWins)];
	mixIn1 =  new double[(int)((winSeg.windowSize)*xFadeWins)];
	mixIn2 =  new double[(int)((winSeg.windowSize)*xFadeWins)];
	bMixOut = new byte[mixOut.length*aus.getSizeInBytes()];

	m_line.addLineListener(this);
	m_line.start();
	randWalk = new RandomWalk();
    }

    /*
     *  constructor:  w/o trigger, variation. uses the default mixer
     * @param winSeg associated class to perform signal segmentation
     * @param aus original segmented audioSample 
     * @param xFadeWin number of windows to crossfade over
     */
    public ByteChannel(WindowedSegmentation winSeg, AudioSample aus,
		       float xFadeWin)
		       
    {
	chunkTime = 1;
	chunkVar = 0;
	period = 1;
	periodVar = 0;
	init( winSeg, aus, xFadeWin);
    }

    /**
     *  constructor: include trigger, variation  
     * @param winSeg associated class to perform signal segmentation
     * @param aus original segmented audioSample 
     * @param transList list of transitions over sound sample
     * @param xFadeWin number of windows to crossfade over
     * @param trig sound is triggered every x seconds
     * @param trigVar trigger variation
     * @param dur everytime sound is triggered, lasts x seconds
     * @param durVar duration variation 
     */
    public ByteChannel(WindowedSegmentation winSeg, AudioSample aus,
		       float xFadeWin, float trig, float trigVar, 
		       float dur, float durVar)
    {
	chunkTime = dur;
	chunkVar = durVar;
	period = trig;
	periodVar = trigVar;
	init( winSeg, aus, xFadeWin);
    }
    
    /**
     * initialize file for recording to 
     * @param filename file name to record to. 
     */
    public void initializeFile(String filename) {
	try {
	    outputFile = new FileOutputStream(filename + ".wav");
	    if (DEBUG) {
		System.out.println("Will write to file: "+ filename + ".wav");
	    }
	    byteArray = new ByteArrayOutputStream();
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * returns the time it will take to play this chunk, in seconds
     * @param startC start position in sample, in bytes
     * @param endC end position in sample, in bytes
     */
    public float calcChunkTime(int startC, int endC) {
	return ((float)(endC - startC))/bytesPerSecond;
    }

    /**
     * keeps track of time played so far in this chunk (max duration)
     * @param startC start position in sample, in bytes
     * @param endC end position in sample, in bytes
     */
    public void updateChunkTime(int startC, int endC) {
	chunkTimeSoFar += calcChunkTime(startC, endC);
    }

    /**
     * sets time played so far in this chunk (max duration)
     * @param startC start position in sample, in bytes
     * @param endC end position in sample, in bytes
     */
    public void setChunkTime(float chT, float chV) {
	chunkTime = chT;
	chunkVar = chV;
	randChunk.setState(chunkTime, chunkVar);
    }

    /**
     * sets period (a chunk will be played every x seconds
     * @param chT time in seconds
     * @param chV variation in seconds
     */
    public void setPeriod(float chT, float chV) {
	period = chT;
	periodVar = chV;
	randPeriod.setState(period, periodVar);
    }

    // rand is the segment number. 
    public void getFirstSeg(int rand) {

	// System.out.println("rand: " + rand);
	try {
	    firstWin =  m_transList[rand][0] +  m_transList[rand][1] 
		- xFadeBytes;
	    // initial write
	    nBytesWritten = 
		m_line.write(audioSample.audioBytes, 
			     m_transList[rand][0],
			     m_transList[rand][1] - xFadeBytes);
	    chunkTimeSoFar = 
		calcChunkTime(m_transList[rand][0], firstWin); 
	    chunkTime = Math.abs(randChunk.nextDouble());

	    if (DEBUG) {
		System.out.println("initial chunk time so far: " 
				   + chunkTimeSoFar); 
		System.out.println("initial chunk time: " + chunkTime); 
	    
	    }
	    period = Math.abs(randPeriod.nextDouble());	    
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }


    /** 
     * write data to sound card
     * @param bOut array of sound data
     * @param bStart byte index to start sound from
     * @param bLength length in bytes to play
     **/
     
    private void writeXBytes(byte [] bOut, int bStart, int bLength) 
    {
	try {
	    nBytesWritten = 
		m_line.write(bOut, bStart, bLength);
	}
	catch (Exception e) {
	    e.printStackTrace();
	}    
    }


    ///// xxxxx .....
    /**
     * @param oldSegmentNum segment number of last segment played
     * @param segmentNumber segment number of new segment to be played     
     **/
    public void playByteSegment(byte[] oldBytes,
			       byte[] newBytes) 
    {
	int segmentEnd = newBytes.length - xFadeBytes;
	int segmentBegin = xFadeBytes;
	
	// frame number of oldSegmentEnd, or number of frames?
	int oldSegmentEnd = oldBytes.length - xFadeBytes;
	
	try {
	    // mix!
	    mixBytes(oldBytes, oldSegmentEnd, xFadeBytes, 
		     newBytes, segmentBegin, xFadeBytes);
	    
	    writeXBytes(bMixOut, 0, bMixOut.length);
	    // System.out.println("pos: write " + (segmentBegin + xFadeBytes) 
	    //	       + " to " + segmentEnd);
	    nBytesWritten = 
		m_line.write(newBytes, xFadeBytes, segmentEnd - segmentBegin );
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
	// file 
	if (outToFile) {
	    try	{
				// System.out.println("out To file");
		byteArray.write(bMixOut, 0, bMixOut.length);
		byteArray.write(newBytes, xFadeBytes,
				segmentEnd - segmentBegin);
	    }
	    catch (Exception e) {
		e.printStackTrace();
	    }
	}
    }

    /**
     * @param oldSegmentNum segment number of last segment played
     * @param segmentNumber segment number of new segment to be played     
     **/
    public void playOneSegment(int oldSegmentNumber, 
			       int segmentNumber)
    {
	int segmentBegin = m_transList[segmentNumber][0];
	int segmentEnd = m_transList[segmentNumber][0] + 
	    m_transList[segmentNumber][1] - xFadeBytes;
	// frame number of oldSegmentEnd, or number of frames?
	int oldSegmentEnd = m_transList[oldSegmentNumber][0] + 
	    m_transList[oldSegmentNumber][1] - xFadeBytes;
	
	if (segmentNumber == (oldSegmentNumber +1)) {
	    // System.out.println("just play!");
	    // System.out.println("writing from " + (segmentBegin-xFadeBytes)
	    //		       + " to " + segmentEnd);
	    try {
		nBytesWritten = 
		    m_line.write(audioSample.audioBytes, 
				 segmentBegin - xFadeBytes,
				 segmentEnd - segmentBegin);
	    }
	    catch (Exception e) {
		e.printStackTrace();
	    }
	    // file 
	    if (outToFile) {
		try	{
		    // System.out.println("out To file");
		    byteArray.write(audioSample.audioBytes,
				    segmentBegin - xFadeBytes, 
				    segmentEnd - segmentBegin);
		}
		catch (Exception e) {
		    e.printStackTrace();
		}
	    }
	    
	}
	else {
	    if (DEBUG) {
		System.out.println("playOneSegment: " + oldSegmentNumber 
				   + " " + segmentNumber);
	    }
	    try {
		// mix!
		mixBytes(audioSample.audioBytes, oldSegmentEnd, xFadeBytes, 
			 audioSample.audioBytes, segmentBegin, xFadeBytes);

		writeXBytes(bMixOut, 0, bMixOut.length);
		// System.out.println("pos: write " + (segmentBegin + xFadeBytes) 
		//	       + " to " + segmentEnd);
		nBytesWritten = 
		    m_line.write(audioSample.audioBytes, 
			     segmentBegin + xFadeBytes, segmentEnd - segmentBegin);
	    }
	    catch (Exception e) {
		e.printStackTrace();
	    }
	    // file 
	    if (outToFile) {
		try	{
				// System.out.println("out To file");
		    byteArray.write(bMixOut, 0, bMixOut.length);
		    byteArray.write(audioSample.audioBytes,
				    segmentBegin + xFadeBytes, 
				    segmentEnd - segmentBegin);
		}
		catch (Exception e) {
		    e.printStackTrace();
		}
	    }
	}
    }


    /**
     * writes given segment to soundcard, no crossfade. 
     * @param segmentNumber  segment number to play. 
     **/
    public void playOneSegment(byte[] seg) 
    {
	int segmentEnd = seg.length - xFadeBytes;

	if (DEBUG) {
	    System.out.println("ByteChannel.playOneSegment() 2 : starting");
	    System.out.println("seg size: " + seg.length);
	}
	try {
	    nBytesWritten = m_line.write(seg, 0, segmentEnd);
	}
	catch (Exception e) {
	    e.printStackTrace();
	}

	if (outToFile) {
	    try	{
		byteArray.write(seg,0, segmentEnd);
	    }
	    catch (Exception e) {
		e.printStackTrace();
	    }
	}
    }

    public void endOfSoundFile() {
	try {	    
	    System.out.println("close file");
	    System.out.println("There are " +  byteArray.size() +
			       " bytes in byteArray");
	    int count = byteArray.size();
	    ByteArrayInputStream bais = new 
		ByteArrayInputStream(byteArray.toByteArray());
	    
	    System.out.println("There are " +  bais.available() +
			       " bytes in bais");
	    
	    // need cound in frames, not bytes!
	    count = count/audioSample.format.getFrameSize();
	    AudioInputStream ais = 
		new AudioInputStream(bais, audioSample.format,count);
	    
	    System.out.println("There are " +  ais.available() +
			       " bytes in this Audioinputstream");
	    
	    nBytesWritten = 
		AudioSystem.write(ais,
				  AudioFileFormat.Type.WAVE,
				  outputFile);
	    if (DEBUG) {
		System.out.println("file1: bytes written: " 
				   + nBytesWritten);
	    }
	    outputFile.close();
	}
	catch (IOException e)
	    {
		e.printStackTrace();
	    }
    }	

    /**
     * main loop to play sound. Also writes to file too, if this is selected.  
     */
    public void run()
    {
	int numSegs = winSeg.segments.size()-1;		
	
	if (DEBUG) {
	    System.out.println("ByteChannel.run(): starting");
	    System.out.println("signal size: " + audioSample.signalCutoff);
	}

	// xx what to do about this? -- will it screw up region stuff? 
	if (invNeedsUpdate) {
	    System.out.println("creating " + numSegs + " segments");
	    segment = new Segment[numSegs];
	    for (int i =0; i < numSegs; i++) {		
		segment[i] = new Segment(i, winSeg);
		segment[i].computeTransitions(noise);
	    }
	    invNeedsUpdate = false;
	}
	
	// only if no regions
	rand = (int)(Math.rint((Math.random()*
				(m_transList.length-1))));
	getFirstSeg(rand);
	
	// main loop
	while (keepGoing) {	    
	    try {
		numSegs = winSeg.segments.size()-1;		
		if (invNeedsUpdate) {		    
		    segment = new Segment[numSegs];
		    for (int i =0; i < numSegs; i++) {		
			segment[i] = new Segment(i, winSeg);
			segment[i].computeTransitions(noise);
		    } 
		    invNeedsUpdate = false;
		    rand = (int)(Math.rint((Math.random()*
					    (m_transList.length-1))));  
		}
		else {	
		    rand = segment[rand].getIndex();
		    if (DEBUG) {
			System.out.println("rand.got: " + rand);
		    }
		}
	    
		// mix!
		mixBytes(audioSample.audioBytes, firstWin, xFadeBytes, 
			 audioSample.audioBytes, 
			 m_transList[rand][0], xFadeBytes);
		
		lastWin =  m_transList[rand][0] + xFadeBytes;

		// write bytes
		writeXBytes(bMixOut, 0, bMixOut.length);
		firstWin = m_transList[rand][0] + m_transList[rand][1] 
		    - xFadeBytes; 		
	    
		if (lastWin < firstWin) { 
		    // write
		    try {
			chunkTimeSoFar += 
			    calcChunkTime(lastWin, firstWin); 
			if (DEBUG) {
			    System.out.println("Chunk Time so far: " 
					       + chunkTimeSoFar);
			    System.out.println("lastWin: " + lastWin + 
					   " firstWin: " + firstWin);
			}
			nBytesWritten = 
			    m_line.write(audioSample.audioBytes, 
					 lastWin, firstWin - lastWin);
		    }
		    catch (Exception e) {
			e.printStackTrace();
			break;
		    }
		    // file 
		    if (outToFile) {
			try	{
				// System.out.println("out To file");
			    byteArray.write(bMixOut, 0, bMixOut.length);
			    byteArray.write(audioSample.audioBytes, lastWin, 
					    firstWin - lastWin);
			}
			catch (Exception e)
			    {
				e.printStackTrace();
			    }
		    }
		}
		if  (chunkTimeSoFar >= chunkTime) {
		    try {
			if (period > chunkTimeSoFar) {
			    if (DEBUG) {
				System.out.println("sleeping for: " 
						   +(long)((period-chunkTimeSoFar)*1000));
			    } 
			    Thread.sleep((long)((period-chunkTimeSoFar)*1000)); 
				// that many milliseconds? 
			}	
		    }
		    catch (InterruptedException e) {
			e.printStackTrace();
		    }
		    chunkTimeSoFar = 0;
		    chunkTime = Math.abs(randChunk.nextDouble());
		    period = Math.abs(randPeriod.nextDouble());
		    if ((gainS != gainM) || (gainM != gainE)) {
			if (DEBUG) {
			    System.out.println("ByteChannel: shifting gain to: " + 
					       gainS + " " + gainM + " " + gainE);
			}
			gainShift.shiftControl(gainS, gainM, gainE,
					       (float)chunkTime);	 
		    }
		    else { 
			gainShift.setControl(gainE);
		    }
		    if (doWalk) {
			System.out.println("do walk");
			// get time. 
			panShift.setControl(randWalk.next(chunkTime));
		    }
		    else if ((panS != panM) || (panM != panE)) {
			panShift.shiftControl(panS, panM, panE, 
					      (float)chunkTime);
		    }
		    else {
			panShift.setControl(panE);
		    }
		}
	    }
	    catch (ArrayIndexOutOfBoundsException e) {
		// re-initialize anything? 
		if (DEBUG) {
		    e.printStackTrace();
		}
	    }
	}
	if (outToFile) {
	    endOfSoundFile();	    
	}	
    }
    
    /**
     * updates transition list. 
     */
    public void updateTransList()
    {
	if (DEBUG) {
	    System.out.println("changing translist");
	}	
	m_transList = winSeg.createTransList();
	invNeedsUpdate = true;
    }


    /**
     * crossfade routine, given bytes, byte positions
     * @param sig byte array of audio signal
     * @param sA byte number to start
     * @param eA number of Bytes to mix
     * @param sB byte number to mix into
     * @param eB number of bytes to mix into
     * I guess eB and eA should be the same??
     */
 
    // change to make it more palatable -- take one frame
    // from each side of the crossfade line. 
    
    void mixBytes(byte[] sigA, int sA, int eA, 
		  byte[] sigB, int sB, int eB) 
    {
	if (DEBUG){
	    System.out.println("mix: " + sA + " to " + (sA+eA) + " and "
			       + sB + " to " + (sB+eB)); 
	}
	
	// convert to double
	
	WavRoutines.getDoubleData(sigA, sA, sA+eA, mixIn1,
				  audioSample.format);
	WavRoutines.getDoubleData(sigB, sB, sB+eB, mixIn2,
				  audioSample.format);
	
	// mix
	mix(mixIn1, 0, mixIn1.length, 
	    mixIn2, 0, mixIn2.length,
	    mixOut);

	// convert back to byte
	WavRoutines.getByteData(mixOut, mixOut.length, bMixOut, 
				audioSample.format);
    }
    
    /**
     * crossfade, given doubles, double positions. 
     */
    public void mix(double [] signalA, int sA, int eA,
  		         double [] signalB, int sB, int eB, 
			 double [] mixOut) 
    {
	// System.out.println("mixOutLength: " + mixOut.length + " eA-sA: " + (eA-sA) + " eB-sB: " + (eB-sB));
	
	double iter = 1/(double)mixOut.length;
	double line = 0;

	for (int i=0; i < mixOut.length; i++) {
	    mixOut[i] = (1-line)*(signalA[sA+i]) + line*(signalB[sB+i]);
	    line += iter;
	}
    }

    /**
     * empty function declared so that this can be a valid LineEvent class
     */
    public void update(LineEvent event)
    {
	if (DEBUG)  {
		System.out.println("AudioPlayer.update(): called with " 
				   + event);
	}
    }
    
    /**
     * needed for lineEvent
     */
    public void startChannel()
    {
	m_line.start();
	// super.start();
    }

    /**
     * start up the sound!
     */ 
    public void startSound() {
	if (DEBUG) {
	    System.out.println("bc.start sound");
	}
	//	if (soundThread == null) {
	soundThread = new Thread(this, "soundThread");
	soundThread.setDaemon(true);
	soundThread.setPriority(9);
	soundThread.start();
	    // }

	keepGoing = true;
	panShift.cancelAll = false;
	gainShift.cancelAll= false;
	// m_line.start();
    }

    /**
     * stop playing!
     */
    public void stopSound() {
	// if (DEBUG) {
	    System.out.println("bc.stop sound");
	    // }
	soundThread = null;
	keepGoing = false;
	panShift.cancelAll = true;
	gainShift.cancelAll= true;
	// m_line.stop();
    }	
}

