/*
 * @(#)CapturePlayback.java	
 *  Reynald Hoskinson
 *  May 19 2001
 */

package sound;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.util.Enumeration;
import java.io.*;
import javax.sound.sampled.*;
import java.net.URL;

import wavelet.*;
import utilities.WavRoutines;

/**
 *  Load an audio file
 *  for streaming playback. Segment a sound into "natural grains" 
 *
 * @version @(#)CapturePlayback.java	1.11	02/02/12
 * @author Reynald Hoskinson
 */
public class CapturePlayback extends JPanel implements ActionListener, 
						       KeyListener 
{
    JFrame f;
    final int bufSize = 16384;
    private static final boolean    DEBUG = false ;

    AudioSample audioSample;
    Playback playback;
    PlayClip playClip = new PlayClip();

    SamplingGraph samplingGraph;
    JButton waveB;

    JTextField textField;
    JTextField segmentField;
    String     fileName = "untitled";
    JSlider threshSlider; 
    JSlider noiseSlider; 
    static final int THRESH_INIT = 25;    //initial threshold

    int wavFilter = 10;
    ByteChannel byteChannel;
    float xFadeWins = 0.25f;
    // keep this *4 !
    WindowedSegmentation separate = new 
	WindowedSegmentation((int)(xFadeWins*4));

    RegionList regionList = new RegionList(10);
    boolean sectionsCurrent = false;

    TopButtons topB;

    public void show() {
	if (f != null) {
	    f.show();
	}
    }

    /**
     * applet stuff 
     */
    public static void showInfoDialog() {
        final String msg = 
            "When running Natural Grains as an applet these permissions\n" +
            "are necessary in order to load/save files and record audio :  \n\n"+
            "grant { \n" +
            "  permission java.io.FilePermission \"<<ALL FILES>>\", \"read, write\";\n" +
            "  permission javax.sound.sampled.AudioPermission \"record\"; \n" +
            "  permission java.util.PropertyPermission \"user.dir\", \"read\";\n"+
            "}; \n\n" +
            "The permissions need to be added to the .java.policy file.";
        new Thread(new Runnable() {
            public void run() {
                JOptionPane.showMessageDialog(null, msg, "Applet Info", JOptionPane.INFORMATION_MESSAGE);
            }
        }).start();
    }

    /**
     * initialize segmentation window
     */
    public void init() {
	f = new JFrame("Segmentation Window");
        f.getContentPane().add("Center", this);
        f.pack();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int w = 850;
        int h = 400;
        f.setLocation(screenSize.width/2 - w/2, screenSize.height/2 - h/2);
        f.setSize(w, h);
    }

    /**
     * Handle the key typed event from the text field. 
     */
    public void keyTyped(KeyEvent e) {
	System.out.println("Key typed: " + e);
    }

    /**
     * Handle the key pressed event from the text field. 
     */
    public void keyPressed(KeyEvent e) {
	System.out.println("KeyPressed: " + e);
    }

    /**
     * Handle the key released event from the text field. 
     */
    public void keyReleased(KeyEvent e) {
        System.out.println("KeyReleased: " + e);
    }

    /**
     * constructor -- makes gui too. 
     */
    public CapturePlayback() {
        setLayout(new BorderLayout());
        EmptyBorder eb = new EmptyBorder(5,5,5,5);
        SoftBevelBorder sbb = new SoftBevelBorder(SoftBevelBorder.LOWERED);
        setBorder(new EmptyBorder(5,5,5,5));

        JPanel p2 = new JPanel();
        p2.setBorder(sbb);
        p2.setLayout(new BoxLayout(p2, BoxLayout.Y_AXIS));

        JPanel buttonsPanel = new JPanel();
        buttonsPanel.setBorder(new EmptyBorder(10,0,5,0));
	topB = new TopButtons(this, regionList);
	// add TopButtons;
	p2.add(topB);
        p2.add(buttonsPanel);

        JPanel samplingPanel = new JPanel(new BorderLayout());
        eb = new EmptyBorder(10,20,20,20);
        samplingPanel.setBorder(new CompoundBorder(eb, sbb));
        samplingPanel.add(samplingGraph = new SamplingGraph(regionList));
	samplingPanel.addKeyListener(this);

        p2.add(samplingPanel);

        JPanel savePanel = new JPanel();
        savePanel.setLayout(new GridLayout(0,2));


	/* COMBOBOX */
	JPanel jumblePanel = new JPanel();

        JLabel wf = new JLabel("Wavelet Filter: ", JLabel.CENTER);
	wf.setVerticalAlignment(JLabel.BOTTOM);
	jumblePanel.add(wf);
	
	JComboBox filterListBox = 
	    new JComboBox(WaveletFilter.WaveletFilterList);
        filterListBox.setSelectedIndex(wavFilter);
	jumblePanel.add(filterListBox);
	
	filterListBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    JComboBox cb = (JComboBox)e.getSource();
		    wavFilter = cb.getSelectedIndex();
            }
        });

	savePanel.add(jumblePanel);

	JPanel thresholdPanel = new JPanel();
	thresholdPanel.add(new JLabel("Segment Threshold: ", JLabel.CENTER));
	threshSlider = new JSlider(JSlider.HORIZONTAL, 0, 100, THRESH_INIT);
	threshSlider.addChangeListener(new SliderListener());
	threshSlider.setMajorTickSpacing(100);
	threshSlider.setMinorTickSpacing(10);
	threshSlider.setPaintTicks(true);
	// threshSlider.setPaintLabels(true);
	threshSlider.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));
	thresholdPanel.add(threshSlider);
	savePanel.add(thresholdPanel);

	/* COMBOBOX */
        JPanel saveTFpanel = new JPanel();
	waveB = addButton("Save WAVE", saveTFpanel, false);
        saveTFpanel.add(new JLabel("File to save:  "));
        saveTFpanel.add(textField = new JTextField(fileName));
        textField.setPreferredSize(new Dimension(140,25));
	
        savePanel.add(saveTFpanel);

	JPanel noisePanel = new JPanel();

	noisePanel.add(new JLabel("segs:"));
	noisePanel.add(segmentField = new JTextField("0"));
	segmentField.setPreferredSize(new Dimension(60,25));

	noisePanel.add(new 
		       JLabel(" Noise: ", JLabel.CENTER));
	noiseSlider = new JSlider(JSlider.HORIZONTAL, 0, 10, 2);
	noiseSlider.addChangeListener(new noiseListener());
	noiseSlider.setMajorTickSpacing(10);
	noiseSlider.setMinorTickSpacing(1);
	noiseSlider.setPaintTicks(true);
	noiseSlider.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));
	noisePanel.add(noiseSlider);

	savePanel.add(noisePanel);
        p2.add(savePanel);
	p2.add(regionList);
        add(p2);
    }


    /**
     * utility function to assist in adding a gui button
     */
    private JButton addButton(String name, JPanel p, boolean state) {
        JButton b = new JButton(name);
        b.addActionListener(this);
        b.setEnabled(state);
        p.add(b);
        return b;
    }

    /**
     *  regular play -- just plays the entire sample, in order
     */
    public void regPlay() {
	// create a new thread. 
	playback.start();
	// also creates a new thread? 
	samplingGraph.start();
    }

    public void playRegion() {
	// xx TODO: check if there are regions defined first
	// if not, possible a dialog box? 
	
	// get current Region - want byteIndex
	int frameStart = regionList.getCurrentlySelectedRegionStart();
	int frameSizeInBytes =audioSample.winIncrement
	    *audioSample.getSizeInBytes();
	playClip.from = frameSizeInBytes*frameStart;
	
	// i'd better make sure segmentB is the end_ of the last segment, 
	// not the beginning. --> sampling graph.  I think so. 
	// add the length of the first clip.
	playClip.clipLength = (regionList.getCurrentlySelectedRegionEnd()
			       *frameSizeInBytes) - playClip.from;
	playClip.start();
	samplingGraph.start();
	
	if (DEBUG) {
	    System.out.println("startframe: " + playClip.from);
	    System.out.println("Numframe: " + playClip.clipLength);
	}
    }

    // actionListener!!
    public void actionPerformed(ActionEvent e) {
	// System.out.println("Wavelet: " + wavFilter);
	
        Object obj = e.getSource();
	if (obj.equals(waveB)) {
            if (byteChannel != null) {
		byteChannel.initializeFile(textField.getText().trim());
		byteChannel.outToFile = true;
	    }
	 // saveToFile(textField.getText().trim(), AudioFileFormat.Type.WAVE);
	}
    }

    public void stopSound() {
	if ((byteChannel != null) && (byteChannel.keepGoing)) {
	    byteChannel.stopSound();
	}
	else {
	    playback.stop();
	    samplingGraph.stop();
	}
    }

    /**
     * gui frontend to load a new soundfile
     */
    public boolean loadFile(String fileName) {
	if (fileName == null) {	    
	    try {
		File file = new File(System.getProperty("user.dir"));
		JFileChooser fc = new JFileChooser(file);
		fc.setFileFilter(new javax.swing.filechooser.FileFilter () {
			public boolean accept(File f) {
			    if (f.isDirectory()) {
				return true;
			    }
			    String name = f.getName();
			    if (name.endsWith(".au") || name.endsWith(".wav") || name.endsWith(".aiff") || name.endsWith(".aif")) {
				return true;
			    }
			    return false;
			}
			public String getDescription() {
			    return ".au, .wav, .aif";
			}
		    });
		separate.blank();
	    
	    if (fc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
		createAudioSample(fc.getSelectedFile(), true);
	    }
	    return true;
	    } catch (SecurityException ex) { 
		CapturePlayback.showInfoDialog();
	    ex.printStackTrace();
	    } catch (Exception ex) { 
		ex.printStackTrace();
	    }
	}
	else {
	    File file = new File(fileName);
	    createAudioSample(file, true);
	}
	return false;
    }	

    private void getReadyToPlay() {
	if (DEBUG) {
	    System.out.println("get ready to play: " + sectionsCurrent);
	}
	if (sectionsCurrent == false) {
	    byteChannel = new ByteChannel(separate, audioSample, 
					  xFadeWins);
	    samplingGraph.updateByteChannel(byteChannel);	
	    byteChannel.noise = noiseSlider.getValue()/(double)10.0; 
	    // initialize segments. 
	    regionList.initialize(byteChannel.noise, separate, topB);
	    sectionsCurrent = true;
	}
    }

    /**
     * reconstitute soundfile by playing segments in semi-random orders. 
     * @param cfg saved data 
     */
    public void randPlay(ConfigParams cfg) {
	boolean writeToFile= false;

	topB.pausB.setEnabled(true);
	topB.playB.setText("Stop");
	
	byteChannel = new ByteChannel(separate, audioSample, xFadeWins, 
				      cfg.trigger, cfg. trigVar, 
				      cfg.duration, cfg.durVar);
	
	samplingGraph.updateByteChannel(byteChannel);
	byteChannel.noise = noiseSlider.getValue()/(double)10.0;	    
	byteChannel.setGain(cfg.gainStart, cfg.gainMid, cfg.gainEnd);
	byteChannel.setPan(cfg.panStart,cfg.panMid, cfg.panEnd);
	byteChannel.panShift.shiftControl(byteChannel.panS,
					  byteChannel.panM,
					  byteChannel.panE,
					  (float)byteChannel.chunkTime);
	byteChannel.gainShift.shiftControl(byteChannel.gainS,
					   byteChannel.gainM,
					   byteChannel.gainE,
					   (float)byteChannel.chunkTime);
	if (byteChannel.outToFile) {
	    byteChannel.initializeFile(textField.getText().trim());
	}
	byteChannel.doWalk = cfg.randWalk;
	byteChannel.startSound();
    }    
    

    /**
     * scramble region
     * 
     */
    public void playScrambledRegions() {
	getReadyToPlay();
	regionList.play(byteChannel);
    }


    public void playOneRegion() {
	getReadyToPlay();
	regionList.playOne(byteChannel);
    }

    /**
     * reconstitute pre-loaded soundfile by 
     *  playing segments in semi-random orders. 
     */
    public void randPlay() {
	getReadyToPlay();
	if (DEBUG) {
	    System.out.println("starting bc..");
	}	
	if (byteChannel.outToFile) {
	    byteChannel.initializeFile(textField.getText().trim());
	}
	byteChannel.startSound();
    }    

    /**
     * given soundfile, create audio sample, update gui components
     * @param file soundfile to open
     * @param updatecomponents true if everything else has to be updated too. 
     */
    public void createAudioSample(File file, boolean updateComponents) {
	if (file != null && file.isFile()) {
            try {
                samplingGraph.errStr = null;
		audioSample = new AudioSample(file);
		audioSample.setSignalCutoff(separate.winIncrement);
		playback = new Playback(this, audioSample, topB);
		
                topB.playB.setEnabled(true);
		topB.separateB.setEnabled(true);
		waveB.setEnabled(true);
		
                if (updateComponents) {
                    samplingGraph.createWaveForm(audioSample, separate, 
						 playback);
		}
            } catch (Exception ex) { 
                samplingGraph.reportStatus(ex.toString());
		ex.printStackTrace();
            }
        } else {
            samplingGraph.reportStatus("Audio file required.");
        }
    }


    /**
     * given URL of soundfile, create audio sample, update gui components
     * @param URL soundfile to open
     * @param updatecomponents true if everything else has to be updated too. 
     */
    public void createAudioSample(URL url) {
            try {
                samplingGraph.errStr = null;
		audioSample = new AudioSample(url);
		audioSample.setSignalCutoff(separate.winIncrement);
		playback = new Playback(this, audioSample, topB);
		
                topB.playB.setEnabled(true);
		topB.separateB.setEnabled(true);
		waveB.setEnabled(true);
		
		samplingGraph.createWaveForm(audioSample, separate, 
						 playback);
            } catch (Exception ex) { 
                samplingGraph.reportStatus(ex.toString());
		ex.printStackTrace();
            }
    }


    /**
     * save configuration to preset file
     * @param name preset filename
     * @param fileType AudioFormat of sound sample. 
     */
    public void saveToFile(String name, AudioFileFormat.Type fileType) {
        if (audioSample.inputStream == null) {
            samplingGraph.reportStatus("No loaded audio to save");
            return;
        }
        // reset to the beginnning of the captured data
        try {
            audioSample.inputStream.reset();
        } catch (Exception e) { 
            samplingGraph.reportStatus("Unable to reset stream " + e);
	    e.printStackTrace();
            return;
        }

        File file = new File(fileName = name);
        try {
            if (AudioSystem.write(audioSample.inputStream, 
				  fileType, file) == -1) {
                throw new IOException("Problems writing to file");
            }
        } catch (Exception ex) { 
	    samplingGraph.reportStatus(ex.toString()); 
	    ex.printStackTrace();
	}
        samplingGraph.repaint();
    }

    /** 
     * Listens to the Threshold slider. 
     */
    class SliderListener implements ChangeListener {
        public void stateChanged(ChangeEvent e) {
            JSlider source = (JSlider)e.getSource();
	    if (separate.occupied) {
		separate.thresholdPercent = source.getValue();
		// System.out.println("Threshold: " + 
		//		       separate.thresholdPercent);
		separate.analyzeTransitions();		    
		
		// separate the red line from the rest of the drawing. 
		if (!source.getValueIsAdjusting()) {		
		    
		    if ((byteChannel != null) &&  
			(byteChannel.keepGoing == true))
			{
			    byteChannel.updateTransList();
			}
		    segmentField.setText((new 
			Long(separate.segments.size())).toString());
		    samplingGraph.updateSeps(separate);
		}
	    }    
	    sectionsCurrent = false;
	}
    }    
    
    /**
     * Listens to the Noise slider. 
     */
    class noiseListener implements ChangeListener {
        public void stateChanged(ChangeEvent e) {
            JSlider source = (JSlider)e.getSource();
	    if (!source.getValueIsAdjusting()) {		
		if (byteChannel != null) {
		    byteChannel.noise = source.getValue()/(double)10.0;	    
		    byteChannel.invNeedsUpdate = true;
		    sectionsCurrent = false;
		}    
	    }
	}
    }
    
    /**
     * Write data to the OutputChannel.
     */
    public class PlayClip implements Runnable {

        SourceDataLine line;
        Thread thread;
	public int from = 0;
	public int clipLength = 0;

	/**
	 *  
	 */
        public void start(int from, int clipLength) {
	    this.from = from;
	    this.clipLength = clipLength;
            samplingGraph.errStr = null;	    
            thread = new Thread(this);
            thread.setName("PlayClip");
            thread.start();
        }

	/**
	 *
	 */
        public void start() {
            samplingGraph.errStr = null;	    
            thread = new Thread(this);
            thread.setName("PlayClip");
            thread.start();
        }

	/**
	 *
	 */
        public void stop() {
            thread = null;
        }
        
	/**
	 *
	 */
        private void shutDown(String message) {
            if ((samplingGraph.errStr = message) != null) {
                System.err.println(samplingGraph.errStr);
                samplingGraph.repaint();
            }
            if (thread != null) {
                thread = null;
                samplingGraph.stop();
                topB.pausB.setEnabled(false);
		if (topB.playRegionB.getText().startsWith("Stop")) {
		    topB.playRegionB.setText("Play Region");
		}
            } 
        }

	/**
	 *
	 */
        public void run() {
            // make sure we have something to play
            if (audioSample.audioBytes == null) {
                shutDown("No loaded audio to play clip back");
                return;
            }

            // define the required attributes for our line, 
            // and make sure a compatible line is supported.

            DataLine.Info info = new DataLine.Info(SourceDataLine.class, 
                audioSample.format);
            if (!AudioSystem.isLineSupported(info)) {
                shutDown("Line matching " + info + " not supported.");
                return;
            }

            // get and open the source data line for playback.

            try {  // line = source data line
                line = (SourceDataLine) AudioSystem.getLine(info);
                line.open(audioSample.format, bufSize);
            } catch (LineUnavailableException ex) { 
                shutDown("Unable to open the line: " + ex);
		ex.printStackTrace();
                return;
            }

            // play back the clip

            // start the source data line
            line.start();
	    try {		    
		line.write(audioSample.audioBytes, from, clipLength);
	    } catch (Exception e) {
		shutDown("Error during playback: " + e);
		e.printStackTrace();
	    }
	    // we reached the end of the stream.  let the data play out, then
            // stop and close the line.
            if (thread != null) {
                line.drain();
            }
            line.stop();
            line.close();
            line = null;
            shutDown(null);
        }
    } // End class Playback
    
    /**
     *   
     */
    public void useTransition(int threshSliderValue, int wavFilterVal, 
			      Transitions ts) {	
	separate.initialize(audioSample);	
	separate.thresholdPercent = threshSliderValue;
	if (DEBUG) {
	    System.out.println("Threshold: " + separate.thresholdPercent);
	}     
	separate.separateWithTransitions(wavFilterVal, ts);
	    segmentField.setText((new 
		Long(separate.segments.size())).toString());
	segRest();
    }

    DefaultBoundedRangeModel rangeModel; 
    JProgressBar progressBar;
    JFrame frameP = new JFrame ("Analyzing Sample");

    /**
     * actually do the segmenting
     */
    public void segmentSample(int threshSliderValue, final int wavFilterVal) 
    {
	rangeModel = new DefaultBoundedRangeModel();
	progressBar = new JProgressBar ( rangeModel );
       
	progressBar.setStringPainted( true );
        progressBar.setString( (int)(progressBar.getPercentComplete()*100) + 
			       "%" );    
        JPanel progressPanel = new JPanel ();
        frameP.getContentPane().add ( progressPanel );
        progressPanel.add ( progressBar );
        frameP.pack();
	
	// size
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int w = 220;
        int h = 50;
        frameP.setLocation(screenSize.width/2 - 
			   w/2, screenSize.height/2 - h/2);
	frameP.setSize(w, h);
        frameP.show();

	separate.initialize(audioSample);
	// separate
	separate.thresholdPercent = threshSliderValue;
	if (DEBUG) {
	    System.out.println("Threshold: " + separate.thresholdPercent);
	}     
	
	final SwingWorker worker = new SwingWorker() {
		public Object construct() {
                     //...code that might take a while to execute is here...
		    separate.separate(wavFilterVal,rangeModel, progressBar);
		    segmentField.setText((new 
			Long(separate.segments.size())).toString());
		    segRest();
		    return separate; // not used
		}
		//Runs on the event-dispatching thread.
		public void finished() {
		    frameP.removeNotify();
		    // initialize byteChannel?
		    if (DEBUG) {
			System.out.println("event dispatching thread: " 
					   + separate.segments.size());
		    }
		}
	    };
	worker.start();        	
    }    

    /**
     *   Segment ? 
     */
    public void segRest() {
	if (separate.segments.size() > 0) {
	    // reload the file if loaded by file
		
	    if (separate.segments.size() > 1) {
		playClip.clipLength = 
		    (((Integer)(separate.segments.get(1))).intValue() +2)*
		    audioSample.format.getFrameSize()*separate.winIncrement;
	    }
	    else {
		playClip.clipLength = 
		    (((Integer)(separate.segments.get(0))).intValue() +2)*
		    audioSample.format.getFrameSize()*separate.winIncrement;
	    }
	    if (DEBUG) {
		System.out.println(": " + playClip.clipLength);
	    }
	    playback = new Playback(this, audioSample, topB);
	    samplingGraph.createWaveForm(audioSample, separate, playback);
	    topB.configureAfterSegmentation();
	    // initialize Regions
	    // regionList.initialize(noiseSlider.getValue()/(double)10.0,
	    //			  separate);
	}
    }

    /**
     *
     */
    public void loadUp(String fileName, boolean segged, int threshSliderVal, 
		       int noiseSliderVal, int wavFilterVal,
		       float period, float periodVar,
		       float chunkTime, float chunkVar,
		       float gainStart, float gainMid, float gainEnd,
		       float panStart, float panMid,  float panEnd) 
    {
	int li;
	String outs;

	init();
	if (DEBUG) {
	    System.out.println("segged: " + segged);
	}
	if ((fileName != null) && (fileName.compareTo("null") != 0))  {
	    if (DEBUG) {
		System.out.println("loading: " + fileName);
	    }
	    try {
		File file = new File(fileName);
		createAudioSample(file, true);		
		if (segged) {
		    separate.occupied = true;
		    // check for object file. 
		    try {
			li = fileName.lastIndexOf(".");
			outs = fileName.substring(0, li);

			FileInputStream objF = 
			    new FileInputStream(outs + ".obj");		
			ObjectInputStream objStream = 
			    new ObjectInputStream(objF);
			Transitions ts = (Transitions)objStream.readObject();
			if (DEBUG) {
			    System.out.println("loaded object " 
					       + outs + ".obj");
			}
			useTransition(threshSliderVal, wavFilterVal, ts);
		    }
		    catch (Exception e) {
			e.printStackTrace();
			System.out.println("no presaved object for "+fileName);
			segmentSample(threshSliderVal, wavFilterVal);
		    }
		}
	    }
	    catch (Exception ex) { 
		samplingGraph.reportStatus(ex.toString());
	    }    
	    threshSlider.setValue(threshSliderVal);
	    noiseSlider.setValue(noiseSliderVal);
	    segmentField.setText((new 
		Long(separate.segments.size())).toString());
	}
    }


    /**
     *
     */
    public void loadURL(URL urlName, boolean segged, int threshSliderVal, 
		       int noiseSliderVal, int wavFilterVal,
		       float period, float periodVar,
		       float chunkTime, float chunkVar,
		       float gainStart, float gainMid, float gainEnd,
		       float panStart, float panMid,  float panEnd) 
    {
	int li;
	String outs;
	String urlString;
	init();
	if (DEBUG) {
	    System.out.println("segged: " + segged);
	}
	createAudioSample(urlName);		
	if (segged) {
	    separate.occupied = true;
	    // check for object file. 
	    try {
		urlString = urlName.toString();
		li = urlString.lastIndexOf(".");
		outs = urlString.substring(0, li);
		URL obj = new URL(outs + ".obj");
		
		InputStream stream = obj.openStream();

		ObjectInputStream objStream = 
		    new ObjectInputStream(stream);
		Transitions ts = (Transitions)objStream.readObject();
		System.out.println("loaded object " + outs + ".obj");
		useTransition(threshSliderVal, wavFilterVal, ts);
	    }
	    catch (Exception e) {
		e.printStackTrace();
		if (DEBUG) {		
		    System.out.println("no presaved object for "+fileName);
		}
		segmentSample(threshSliderVal, wavFilterVal);
	    }
	}
	/*    }
	      catch (Exception ex) { 
	      samplingGraph.reportStatus(ex.toString());
	      }    
	*/
	threshSlider.setValue(threshSliderVal);
	noiseSlider.setValue(noiseSliderVal);
	segmentField.setText((new 
			      Long(separate.segments.size())).toString());
    }
    
    public static void main(String s[]) {
        CapturePlayback capturePlayback = new CapturePlayback();
	capturePlayback.init();
        capturePlayback.f.show();

	// process command line args. 
	if (s.length > 0) {
	    File file = new File(s[0]);
	    if (DEBUG) {
		System.out.println("loading filename: " + file.toString());
	    }
	    capturePlayback.createAudioSample(file, true);
	}
    }
} 
