package sound;

import  javax.sound.sampled.FloatControl;
import  javax.sound.sampled.Mixer;
import  javax.sound.sampled.SourceDataLine;
import java.util.Timer;
import java.util.TimerTask;
import java.awt.Toolkit;

/**
 * provides continous shifting between start and end parameters. 
 * replacement for JavaSound's shift, which doesn't work. 
 */

public class MyShift 
{
    private FloatControl m_control;
    private float        toPan;
    private float        midPan;
    private float        fromPan;

    Toolkit toolkit;
    Timer timer;

    float precision;
    float numUpdate;
    float updatePeriod;
    
    float chunkTime;
    float soFar;
    int updateSoFar;

    boolean cancelAll = false;

    static boolean DEBUG = false;

    public MyShift(SourceDataLine m_line, FloatControl.Type ftc) 
    { 	
	if (m_line.isControlSupported(ftc))
	{
	    m_control = 
		(FloatControl)m_line.getControl(ftc);	    
	    if (DEBUG) {
		System.out.println("max balance: " + 
				   m_control.getMaximum());
		System.out.println("min balance: " + 
				   m_control.getMinimum());
		System.out.println("balance precision: " + 
				   m_control.getPrecision());
	    }
	}
	else {
	    System.out.println(ftc + " is not supported");
	}

	toolkit = Toolkit.getDefaultToolkit();
	timer = new Timer();
    }

    public float getControl()
    {
	if (hasControl())
	{
		return m_control.getValue();
	}
	else
	{
		return 0.0F;
	}
    }

    public boolean hasControl()
    {
	return m_control != null;
    }
    
    public float getPrecision()
    {
	if (hasControl())
	    {		
		return m_control.getPrecision();
	    }
	else
	    {
		return 0.0F;
	    }
    }

    
    public float getMaximum()
    {
	if (hasControl())
	    {
		return m_control.getMaximum();
	    }
	else
	    {
		return 0.0F;
	    }
    }
    
    public float getMinimum()
    {
	if (hasControl())
	    {
		return m_control.getMinimum();
	    }
	else
	    {
		return 0.0F;
	    }
    }
    

    public void setControl(float f) {
	if (hasControl())
	{	
	    if (chunkTime == 0) {
		m_control.setValue(f);
	    }
	}	
    }

    // chunkTime is in seconds. 
    public void shiftControl(float fPanS, float fPanM, float fPanE, 
			     float chunkTime)
    {
	if (DEBUG) {
	    System.out.println("ShiftControl: setting control from " + fPanS 
			       + " to " + fPanM + " to " + fPanE + " over " 
			       + (int)(chunkTime*1000)  + " ms ");  
	}
	//from ms to s
	chunkTime = chunkTime*1000;  // half each. 
	fromPan = fPanS;
	midPan  = fPanM;
	toPan  = fPanE;
	
	if (hasControl())
	{	
	    if (chunkTime == 0) {
		m_control.setValue(fPanE);
	    }
	    precision = m_control.getPrecision();

	    // JAVASOUND BUG???  - gain precision is -40
	    if (precision < 0) {
		precision = 1;
	    }

	    numUpdate = Math.abs((fromPan-midPan)/precision) +
		Math.abs((midPan-toPan)/precision);
	    updatePeriod = chunkTime/numUpdate;

	    if (DEBUG) {
		System.out.println("precision: " + precision);
		System.out.println("numUpdate: " + numUpdate);
		System.out.println("updatePeriod " + updatePeriod);
	    }

	    try {
		timer = new Timer();
		timer.scheduleAtFixedRate(new PanTask(),
					  (long)0,        //initial delay
					  (long)(updatePeriod));  
		//subsequent rate
		updateSoFar = 0;
	    }
	    catch (IllegalArgumentException e) {
	    }
	}
    }

    class PanTask extends TimerTask 
    {
	boolean justStart = true;
	boolean justPast = false;
	int dir=1;
	float u;

	public void run() 
	{
	    if (DEBUG) {
		System.out.println("updates so far: " + updateSoFar);
		System.out.println("numUpdate: " + updateSoFar);
		System.out.println("justStart: " + justStart);
		System.out.println("justPast: " + justPast);
	    }
	    
	    if (justStart) {
		if ( fromPan > midPan) {
		    dir = -1;
		}
		
		if (updateSoFar < (numUpdate/2)) {
		    u = fromPan + dir*updateSoFar*precision;
		    m_control.setValue(u);
		    // System.out.println("setting pan to " + u);
		    updateSoFar++;				
		}
		else {
		    justStart = false;
		    justPast = true;
		}
		// System.out.println("part 1");
	    }
	    else  if (justPast) {
		if ( midPan > toPan) {
		    dir = -1;			
		}
		else { 
		    dir = 1;
		}
		// System.out.println("past midpoint " + dir);  
		updateSoFar =0;
		justPast = false;
		// System.out.println("part 2");
	    }
	    else if (!justPast) {
		if (updateSoFar < (numUpdate/2)) {
		    u = midPan + (dir*updateSoFar*precision);
		    //  System.out.println("setting pan to " + u);
		    m_control.setValue(u);
		    updateSoFar++;		
		    // System.out.println("part 3");
		}
		else {
		    //  System.out.println("timer.cancel");
		    timer.cancel();
		}
	    }
	    
	    if (cancelAll) {
		// System.out.println("timer.cancel");
		timer.cancel();
	    }
	}

    }
}
