package sound;

import	java.io.*;
import  cern.jet.random.engine.DRand;
import  cern.colt.Arrays;
import  utilities.SampleWithoutReplacement;

/** 
 *  This class will include faculties for per subject Testing. 
 *  Input:
 *     - name of file to read sample names from 
 *        - sample names are two per line - one for the original, one 
 *	   for the mixed. 
*/

public class PlaySet
{
    static final int files = 7;   // same as types
    static final int reps = 3;

    String [] orig =  new String[files];
    String [] mixed = new String[files];
    PlayTwo pt [] = new PlayTwo[files];

    String testFile;
    String testFile_mixed;
    PlayTwo playTest;

    int [] tests;

    int ct=0;
    File sampleNames;
    String filePrefix = "/project/reynald/thesis/testing/samples/";
    Archiver ar;
    
    DRand drand = new DRand(new java.util.Date());

    /*
      Input:
      Name
      
      Log:
      
      Randomize:
      Order (Real, Synth)
      about 3.5 seconds. 

      At top:
      Name:
      
      Soundfiles tested (denoted by number, put number at top). 
    */

    public PlaySet(String logname, File sampleNames) {
	int i;
	ar = new Archiver(logname);	
	this.sampleNames = sampleNames;

	for (i = 0; i < files; i++ ) {
	    orig[i]  = new String();
	    mixed[i] = new String();
	}

	// updates the variables orig, mixed. 
	readTwoStringsPerLine(sampleNames);

	for (i = 0; i < files; i++ ) {
	    pt[i] = new PlayTwo(ar, orig[i], mixed[i]);
	}
	playTest = new PlayTwo(ar, testFile, testFile_mixed);
	ar.pauseUntilKeyPress();
	playTest.playABit(true);
	ar.pauseUntilKeyPress();
	ar.writeInto("\n");
    }


    public void playPracticePairs() {
	// get a random number, *numSamples for the sample to play
	// another sample: < 0.5, right, > 0.5, wrong. 
	

    }

    public static void main(String[] args) {
	if (args.length < 1 ) {
	    System.out.println("Usage: PlaySet <Subject name>");
	    System.exit(0);
	}
	
	String logname = new String(args[0] + ".log");
	System.out.println("saving to logname: " + logname);

	// read from testfile.
	String testFile = "/grads1/reynald/thesis/testing/testfile";
	if (args.length > 1) {
	    testFile = args[0];
	}
	File sampleNames = new File(testFile);
	PlaySet ps = new PlaySet(logname, sampleNames);
	
	// mix everything up!
	ps.mixUp();	
	System.out.println("order of tests: " + Arrays.toString(ps.tests));
	
	// play sounds, get results. 
	ps.playSounds();
		   
	ps.finish();
	// set up log file, write start. should do this in constructor. 
    }
    
    public void mixUp () {
	
	int [] possibles = new int[files*reps*2];
	// fill up. 
	int ct=0;

	for ( int i = 0; i < files; i++) {
	    for (int j = 0; j < reps; j++) {
		possibles[ct] = 10*i;
		ct++;
		possibles[ct] = 10*i + 1;
		ct++;
	    }
	}
	System.out.println("order before mixup: " + Arrays.toString(possibles));

	// mix up
	tests = SampleWithoutReplacement.getVariation(possibles, drand);
    }


    public void playSounds() {
	int sample;
	int order;

	for (int i=0; i < tests.length; i++ ) {
	    // decode what test. 
	    System.out.println(i + " tests[i]/10: " + tests[i] + " "  + tests[i]/10);
	    
	    sample = (int)Math.floor((long)(tests[i] / 10));
	    order  = tests[i]%10;
	    System.out.println("Playing sample: " + sample 
			       + " in order " + order);
	    ar.writeInto(new String(sample + " " ));
	    playPair(sample, order);	    
	}	
    }

    public void finish() {
	for (int i=0; i < ct; i++ ) {
	    pt[i].finish();
	}
	System.out.println("PlaySet.main(): before ar.finish");
	ar.finish();

	System.out.println("PlaySet.main(): before exit");
	System.exit(0);
    }
    

    public void playPair (int sample, int order) {

	if (order == 0) {
	    pt[sample].playABit(false);
	} 
	else {
	    pt[sample].playABit(true);
	} 	    
	ar.writeResult();
    }


    public void readTwoStringsPerLine(File fn) {
	// remember number of files in input file. 

	String buf;
	try { 
	    Reader in = new FileReader(fn);
	    BufferedReader bf = new BufferedReader(in);

	    while ((bf != null) && (ct < files))  {
		buf = bf.readLine();
		if (buf != null) {
		    StringReader str = new StringReader(buf);
		    orig[ct] = filePrefix + getUntilSpace(str);
		    mixed[ct] = filePrefix + getUntilSpace(str);
		    ar.writeInto(ct + " "  + orig[ct] 
				 + " " + mixed[ct] + "\n");
		    ct++;
		    // now test
		}
		else {
		    break;
		}
	    }
	    // get testfile. 
	    buf = bf.readLine();
	    if (buf != null) {
		StringReader str = new StringReader(buf);
		testFile = filePrefix + getUntilSpace(str);
		testFile_mixed = filePrefix + getUntilSpace(str);
	    }
	}
	catch (IOException e) {
	    System.out.println("1Error: file not readable");
	}
    }

    // returns index of space 
    public String getUntilSpace(StringReader str) {	
	int c;
	String s = "";
	try {
	    while (((c = str.read()) != ' ') && (c != -1)) {
		s += (char)c;
	    }
	}
	catch (IOException e) {
	    System.out.println("2Error: file not readable");
	    e.printStackTrace();
	}
	return s;
    }	
}
