package sound;

import	java.io.File;
import	java.io.InputStream;
import	java.io.IOException;

import	javax.sound.sampled.AudioFormat;
import	javax.sound.sampled.AudioInputStream;
import	javax.sound.sampled.AudioSystem;
import	javax.sound.sampled.LineUnavailableException;
import	javax.sound.sampled.LineListener;
import	javax.sound.sampled.LineEvent;
import	javax.sound.sampled.Mixer;
import	javax.sound.sampled.SourceDataLine;

import	javax.sound.sampled.DataLine;
import	javax.sound.sampled.FloatControl;
import	javax.sound.sampled.BooleanControl;

import  cern.jet.random.engine.DRand;
import  cern.colt.Arrays;

/**
 * plays random bits of two soundfiles
 * used for user testing
 */

public class PlayTwo
{
    private static boolean              DEBUG = true;
    int                                 m_waitTime   = 5000;  // ms
    private static float                duration = 4.0F;  //seconds

    public SourceDataLine  m_line1, m_line2;
    File                audioFile;
    File	        audioFile2;
    AudioInputStream	audioInputStream = null;
    AudioInputStream	audioInputStream2 = null;
    AudioFormat         audioFormat, audioFormat2;
    float               bytesPerSecond; 
    int                 total_duration_bytes1, total_duration_bytes2;
    int                 duration_bytes;
    DRand               drand = new DRand(new java.util.Date());	
    Archiver            ar;

    byte []             clip1;
    byte []             clip2;

    public static void main(String[] args) {
	
	if (args.length < 3) {
	    System.out.println("PlayTwo: usage:");
	    System.out.println("\tjava PlayTwo <logname> <soundfile1> <soundfile2> ");
	    System.exit(1);
	}

	Archiver ar = new Archiver(args[0]);
	PlayTwo playTwo = new PlayTwo(ar, args[1], args[2]);
	playTwo.ar.writeInto("Testing\n");
	playTwo.playABit(true);
	playTwo.ar.writeResult();
	playTwo.finish();
    }

    public int chop(int num, int roundTo) {
	System.out.println("got: " + num + " round to: " + Math.floor((num/roundTo))*roundTo);
	return (int)Math.floor((num/roundTo))*roundTo;
    }


    public PlayTwo(Archiver ar, String aud1, String aud2) {
	this.ar = ar;
	setupAudioStream(aud1, aud2);
    }

    // boolean controls which first. 
    public void playABit(boolean inOrder) {
	// want it to only start on evens. 

	System.out.println("duration bytes: " + duration_bytes);

	// calculate starting point here. 
	float start    = drand.nextFloat();
	System.out.println("start: " + start);
	int start_bytes =  (int)((total_duration_bytes1 
				  - duration_bytes) * start);	

	start_bytes = chop(start_bytes, audioFormat.getFrameSize());
	start    = drand.nextFloat();
	int start_bytes2 =  (int)((total_duration_bytes2 
				   - duration_bytes) * start);	
	start_bytes2 = chop(start_bytes2, audioFormat2.getFrameSize());	
	
	ar.writeOrder(inOrder);
		
	if (inOrder) {
	    System.out.println("play inorder");
	    writeClipToLine(clip1, start_bytes, duration_bytes, m_line1);
	    try {
		Thread.sleep(m_waitTime);
	    }
	    catch (InterruptedException e) {
	    }	    
	    writeClipToLine(clip2, start_bytes2, duration_bytes, m_line2);
	}
	else {
	    System.out.println("play out order");
	    writeClipToLine(clip2, start_bytes2, duration_bytes, m_line2);
	    try {
		Thread.sleep(m_waitTime);
	    }
	    catch (InterruptedException e) {
	    }
	    writeClipToLine(clip1, start_bytes, duration_bytes, m_line1);
	}
    }	

    // given two soundfile names, sets up everything in order to play them. 
    private void setupAudioStream(String strFileName1, String strFileName2)
    {
	audioFile    = new File(strFileName1);
	audioFile2   = new File(strFileName2);	
	if (DEBUG) {
	    System.out.println("Trying to open streams: " + strFileName1 
			       + " and " + strFileName2);
	}

	try {
	    audioInputStream  = AudioSystem.getAudioInputStream(audioFile);
	    audioInputStream2 = AudioSystem.getAudioInputStream(audioFile2);
     	}
	catch (Exception e) {
	    e.printStackTrace();
	    System.exit(1);
	}
	audioFormat  = audioInputStream.getFormat();
	audioFormat2  = audioInputStream2.getFormat();
	
	if ((audioInputStream != null) && (audioInputStream2 != null)) {
	    bytesPerSecond 
		= audioFormat.getFrameSize() * audioFormat.getFrameRate(); 
	    
	    total_duration_bytes1 = (int)(audioInputStream.getFrameLength()
					 *audioFormat.getFrameSize());

	    total_duration_bytes2 = (int)(audioInputStream2.getFrameLength()
					  *audioFormat2.getFrameSize());
	    
	    // already chopped
	    duration_bytes = (int) (duration * bytesPerSecond);
	    

	    ar.writeInto("duration of total sample: " 
			 + total_duration_bytes1/bytesPerSecond + " and " + 
			 total_duration_bytes2/bytesPerSecond + "\n");
	    System.out.println("duration of total sample: "  +  
			       total_duration_bytes1/bytesPerSecond + " and " 
			       +  total_duration_bytes2/bytesPerSecond + "\n");
	    System.out.println("bytesPerSecond: " + bytesPerSecond);

	    Mixer mixer    = AudioSystem.getMixer(null);	    	
	    try {
		DataLine.Info   info = new DataLine.Info(SourceDataLine.class,
							 audioFormat);
		m_line1 = (SourceDataLine) mixer.getLine(info);
		m_line1.open(audioFormat, m_line1.getBufferSize());
		m_line1.start();			

		info = new DataLine.Info(SourceDataLine.class,
					 audioFormat2);
		m_line2 = (SourceDataLine) mixer.getLine(info);
		m_line2.open(audioFormat2, m_line2.getBufferSize());
		m_line2.start();			
	    }
	    catch (Exception e) {
		e.printStackTrace();
	    }
	    clip1 = new byte[total_duration_bytes1];
	    clip2 = new byte[total_duration_bytes2];
	    
	    int n;
	    try {	    
		n = audioInputStream.read(clip1, 0, total_duration_bytes1);
		System.out.println("read " + n + " into clip1");
		n = audioInputStream2.read(clip2, 0, total_duration_bytes2);
		System.out.println("read " + n + " into clip2");
	    }		
	    catch (Exception e) {
		e.printStackTrace();
		System.exit(1);
	    }	    
	}	
	else {
	    System.out.println("Warning: Can't open soundfiles");
	}
    }
    
    public void finish() {
	if (DEBUG)   {
	    System.out.println("PlayTwo.main(): before close");
	}
	// m_line.close();
    }

    private void writeClipToLine(byte [] clip, int start, int duration, SourceDataLine line) {

	int nBytesWritten = 0;
	int tot = 0;
	try {	    
	    System.out.println("write " + " start " + start 
			       + " duration " + (duration - tot));
	    nBytesWritten = line.write(clip, start, (duration - tot));
	    tot += nBytesWritten;
	}
	catch (Exception e) {
	    e.printStackTrace();
	}		
	if (DEBUG) {
	    System.out.println("PlayTwo.writeClipToLine(): written to SourceDataLine (bytes): " 
			       + nBytesWritten);
	}
	// m_line.drain();	    
    }
}
