package sound;
import  java.io.*;
import  javax.sound.sampled.*;

/**
 * Write data to the OutputChannel.
 */
public class Playback implements Runnable {

    SourceDataLine line;
    Thread thread;
    CapturePlayback cap;
    AudioSample audioSample;
    TopButtons topB;
    final int bufSize = 16384;

    public Playback(CapturePlayback cp, AudioSample aus,
		    TopButtons top) {
	topB = top;
	cap = cp;
	audioSample = aus;
    }

    public void start() {
	cap.samplingGraph.errStr = null;
	thread = new Thread(this);
	thread.setName("Playback");
	thread.start();
    }

    public void stop() {
	thread = null;
    }
    
    private void shutDown(String message) {
	if ((cap.samplingGraph.errStr = message) != null) {
	    System.err.println(cap.samplingGraph.errStr);
	    cap.samplingGraph.repaint();
	}
	if (thread != null) {
	    thread = null;
	    cap.samplingGraph.stop();
	    topB.pausB.setEnabled(false);
	    topB.playB.setText("Play");
	} 
    }

    public void run() {

	AudioInputStream playbackInputStream = 
	    audioSample.getAudioInputStream();
	
	// make sure we have something to play
	if (playbackInputStream == null) {
	    shutDown("No loaded audio to play back");
	    return;
	}
	
	// define the required attributes for our line, 
	// and make sure a compatible line is supported.

	DataLine.Info info = new DataLine.Info(SourceDataLine.class, 
					       audioSample.format);
	if (!AudioSystem.isLineSupported(info)) {
	    shutDown("Line matching " + info + " not supported.");
	    return;
	}
	
	// get and open the source data line for playback.
	
	try {
	    line = (SourceDataLine) AudioSystem.getLine(info);
	    line.open(audioSample.format, bufSize);
	} catch (LineUnavailableException ex) { 
	    shutDown("Unable to open the line: " + ex);
	    return;
	}
	
	int frameSizeInBytes = audioSample.format.getFrameSize();
	int bufferLengthInFrames = line.getBufferSize() / 8;
	int bufferLengthInBytes = bufferLengthInFrames * frameSizeInBytes;
	byte[] data = new byte[bufferLengthInBytes];
	int numBytesRead = 0;
	
	// start the source data line
	line.start();
	while (thread != null) {
	    try {
		if ((numBytesRead = playbackInputStream.read(data)) == -1) {
		    break;
		}
		int numBytesRemaining = numBytesRead;
		while (numBytesRemaining > 0 ) {
		    numBytesRemaining -= line.write(data, 0,numBytesRemaining);
                    }
	    } catch (Exception e) {
		shutDown("Error during playback: " + e);
		break;
	    }
	}
	// we reached the end of the stream.  let the data play out, then
	// stop and close the line.
	if (thread != null) {
	    line.drain();
	}
	line.stop();
	line.close();
	line = null;
	shutDown(null);
    }
} // End class Playback
