package sound;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.util.Vector;
import utilities.*;
import  cern.jet.random.engine.DRand;

/**
 * characterises a number of segments
 * that define a temporal/spectral structure
 * we'd like to preserve & fool around with. 
 */
public class Region extends JPanel 
{
    /**
     * the segment start/end 
     **/
    int segmentA, segmentB;
    JFrame f;

    int regionNumber;
    // list of percentages for all other vectors, for use in
    // determining likelihood of transition between this
    // and the others. 

    // store Jsliders. 
    Vector otherRegions = new Vector();
    double []  scratch = new double[10];

    int defaultOtherRegion = 50;
    JPanel otherRegionsPanel = new JPanel();

    float startWeight  = 1F;
    float middleWeight = 1F;
    float endWeight    = 1F;
    float startJointWeight = 5F;
    float ampWeight    = 1.0F;
    float lengthWeight = 1.0F;

    JTextField startWeightField;
    JTextField middleWeightField;
    JTextField endWeightField;
    JTextField startJointWeightField;
    JTextField ampWeightField;
    JTextField lengthWeightField;

    Segment [] segment;
    ByteChannel byteChannel;
    boolean keepGoing = false;
    DRand drand;

    double [] segmentBuffer = new double[100000]; // HACK!!!
    byte [] segmentByteBuffer;
    byte [] oldSegmentByteBuffer;
    /*
    boolean                dbg                = false; 
    FileOutputStream       outputStream;
    private File           file;
    private String []      outputString;
    */

    // various spectral characteristics we'd like to 
    // have, modify. 

    static final boolean DEBUG = false;

    public void initFrame(int regionNumber) {
	f = new JFrame("Region Number " + regionNumber);
        f.getContentPane().add("Center", this);
        f.pack();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int w = 425;
        int h = 400;
        f.setLocation(screenSize.width/2 - w/2, screenSize.height/2 - h/2);
        f.setSize(w, h);
    }

    public int nextRegion() {
	double sumI = 0;
	// recalculate every time. 
	// determine normalizing factor. 
	int i;
	int nextRegion;
	for  (i = 0; i < otherRegions.size(); i++) {
	    // get slider value;
	    sumI+= ((JSlider)otherRegions.get(i)).getValue();
	}
	
	scratch[0] = ((JSlider)otherRegions.get(0)).getValue()/sumI;
	for (i = 1; i < otherRegions.size(); i++) {
	    scratch[i] = ((JSlider)otherRegions.get(i)).getValue()/sumI + 
		scratch[i-1];
	    // System.out.println("scratch: " + scratch[i]);
	}	    
	double dd = drand.nextDouble();
	System.out.println("dd: " + dd);
	nextRegion = Search.binarySearch(scratch, otherRegions.size(), dd);
	System.out.println("next region: " + nextRegion);
	return nextRegion;
    }

    public void setOtherRegions(int numRegions) {	
	buildOtherRegionsPanel(numRegions);
	// anything else? 
    }
    
    private int getSegmentToBePlayed(int segmentIndex) {
	int numSegments = segmentB - segmentA;
	
	if (segmentIndex < numSegments) {
	    return segmentA + segmentIndex;
	}
	else return 0;
    }

    /**
     * sets the weights by looking at the text fields. 
     */
    public void setWeights() {
	startWeight = 
	    new Float(startWeightField.getText().trim()).floatValue();
	middleWeight = 
	    new Float(middleWeightField.getText().trim()).floatValue();
	endWeight = 
	    new Float(endWeightField.getText().trim()).floatValue();
	startJointWeight = 
	    new Float(startJointWeightField.getText().trim()).floatValue();
	lengthWeight = 
	    new Float(lengthWeightField.getText().trim()).floatValue();
	ampWeight = 
	    new Float(ampWeightField.getText().trim()).floatValue();
    }


    // effect segment with amplitude changing, etc. 
    public void processSegment(int oldSeg, int newSeg) {

	float ratio;
	// put oldSegment into segmentBuffer. 
	// !-- deal with crossfading!
	int startInBytes = byteChannel.m_transList[newSeg][0];
	int lengthInBytes = byteChannel.m_transList[newSeg][1];

	int nSamples = 
	    WavRoutines.getDoubleData(byteChannel.audioSample.audioBytes,
				      startInBytes, lengthInBytes + startInBytes,
				      segmentBuffer, 
				      byteChannel.audioSample.format);
	
	// determine level 
	// already stored somewhere  . . 
	if (DEBUG) {
	    System.out.println("newSeg.avgAmplitude: " + 
			       byteChannel.segment[newSeg].avgAmplitude);
	    System.out.println("old.avgAmplitude: " + 
			       byteChannel.segment[oldSeg].avgAmplitude);
	}
	ratio = (float) 
	    (byteChannel.segment[oldSeg].avgAmplitude/
	     byteChannel.segment[newSeg].avgAmplitude);

	// amplify it to new volume -- watch out for clipping
	/* doesn't work right now -- produces loud noises 
	   sometimes because it normalizes relative silence into
	   a loud sound. 
	   Since I don't, this routine could sure be optimized.
	*/

	// WavRoutines.normalizeTo(segmentBuffer, nSamples, ratio);	
		
	// what is the length of this segment? 
	// xxx -- turn this into a static buffer, the size of 
	// the largest segment
	segmentByteBuffer = new byte[lengthInBytes];
	//convert back to bytes?  -> can I do this operation on bytes? nah.
	WavRoutines.getByteData(segmentBuffer, nSamples, segmentByteBuffer,
				byteChannel.audioSample.format);
    }

    /**
     *  plays a replacement for each segment in the region      
     **/

    // note - I have to crossfade between regions too!
    public void play(ByteChannel bc, Segment [] segList) {
	keepGoing = true;
	byteChannel = bc;
	segment = segList;
	int segmentNum = getSegmentToBePlayed(0);
	int numSegments = segmentB - segmentA;
	int segToPlay = 0;
	int lastSegment;
	System.out.println("numSegments in Region " + regionNumber 
			   + ": " + numSegments);
	setWeights();
	/*
	  System.out.println("weights. start: " +  startWeight + " middle: " + 
	  middleWeight + " end: " + endWeight + " length: "
	  + lengthWeight + " amp: " + ampWeight);
	*/

	// replace first segment
	System.out.println("segment index: " + segment[segmentNum].m_Index);

	segment[segmentNum].computeTransitions(byteChannel.noise,
					       startWeight,
					       middleWeight,
					       endWeight, null, 0,
					       lengthWeight,
					       ampWeight); 

	segToPlay = segment[segmentNum].getIndex();
	// segToPlay = segmentNum;

	System.out.println("1. replacing segment " + segmentNum + 
			   " with " + segToPlay);

	// process segment (amplitude, segment morphing)
	processSegment(segmentNum, segToPlay);
	
	// play it -- minus mixBytes.  	
	byteChannel.playOneSegment(segmentByteBuffer);
	oldSegmentByteBuffer = segmentByteBuffer;
	// now play rest of segments in Region. 
	// ? keep track of sequence? 
	// what holger said about mixing up repeated iterations 
	// could be interesting. 
	System.out.println("keepGoing: " + keepGoing); 
	
	for (int i=1; (i < numSegments) && (keepGoing == true); i++) {
	    // get joint from last segment.
	    lastSegment = segToPlay;
	    double [] startJoint = segment[lastSegment].getJointTransitions();
	    // compute replacement using this info, and the 
	    // original segment num
	    //  System.out.println("using jointT's of : " + lastSegment);

	    segmentNum = getSegmentToBePlayed(i);

	    // commented this out! - changed bytechannel.noise to 0
	    segment[segmentNum].computeTransitions(0,
						   startWeight,
						   middleWeight,
						   endWeight,
						   startJoint, 
						   startJointWeight,
						   lengthWeight,
						   ampWeight); 
	    segToPlay = segment[segmentNum].getIndex();	    
	    // segToPlay = segmentNum;
	    processSegment(segmentNum, segToPlay);	    
	    System.out.println("replacing segment " + segmentNum + 
			       " with " + segToPlay);
	    byteChannel.playByteSegment(oldSegmentByteBuffer, 
					segmentByteBuffer);
	    oldSegmentByteBuffer = segmentByteBuffer;
	}
    }

    public JSlider createSlider(int start, int end, int defaultValue) {
	JSlider j = new JSlider(JSlider.HORIZONTAL, 
				start, end, defaultValue);
	// j.addChangeListener(new SliderListener());
	j.setMajorTickSpacing(50);
	j.setMinorTickSpacing(10);	
	j.setPaintTicks(true);
	j.setPaintLabels(true);
	return j;
    }

    public void addOtherRegion() {
	int placeElementAt = otherRegions.size();
	otherRegions.add(createSlider(0,100, defaultOtherRegion));
	otherRegionsPanel.add(new JLabel("Region " + placeElementAt,
					 JLabel.CENTER));
	otherRegionsPanel.add((JSlider)otherRegions.get(placeElementAt));
    }

    public void deleteOtherRegion(int regionNum) {
	// all this done in RegionList
	otherRegions.removeElementAt(regionNum);
	otherRegionsPanel.remove(regionNum);
    }

    private JPanel makeWeightPanel() {
	
        JPanel wp = new JPanel();       
	wp.add(new JLabel("Weights"), BorderLayout.NORTH);
	
	JPanel inf = new JPanel();
	inf.setLayout(new GridLayout(0,2));	
	inf.add(new JLabel("start"));
	inf.add(startWeightField = new JTextField(Float.toString(startWeight)));
	startWeightField.setPreferredSize(new Dimension(40,25));

	inf.add(new JLabel("middle"));
	inf.add(middleWeightField = new JTextField(Float.toString(middleWeight)));
	middleWeightField.setPreferredSize(new Dimension(40,25));

	inf.add(new JLabel("end"));
	inf.add(endWeightField = new JTextField(Float.toString(endWeight)));
	endWeightField.setPreferredSize(new Dimension(40,25));

	inf.add(new JLabel("joint"));
	inf.add(startJointWeightField = new JTextField(Float.toString(startJointWeight)));
	startJointWeightField.setPreferredSize(new Dimension(40,25));	

	inf.add(new JLabel("amp"));
	inf.add(ampWeightField = new JTextField(Float.toString(ampWeight)));
	ampWeightField.setPreferredSize(new Dimension(40,25));	

	inf.add(new JLabel("length"));
	inf.add(lengthWeightField = new JTextField(Float.toString(lengthWeight)));
	lengthWeightField.setPreferredSize(new Dimension(40,25));	

	wp.add(inf, BorderLayout.CENTER);		
	return wp;
    }

    private void buildOtherRegionsPanel(int numRegions) {
	
	// clear old sliders off panel?
	otherRegions.removeAllElements();
	otherRegionsPanel.removeAll();
	otherRegionsPanel.add(new JLabel("---Region Transitions---", JLabel.CENTER));
	

	for (int i = 0; i < numRegions; i++) {
	    // fill otherRegionsPanel
	    otherRegions.add(createSlider(0, 100, defaultOtherRegion));
	    otherRegionsPanel.add(new JLabel("Region " + i, JLabel.CENTER));
	    otherRegionsPanel.add((JSlider)otherRegions.get(i));
	}
    }

    /**
     * Region Constructor
     * @param regionNumber index of this region in RegionList
     * @param start index to the WindowSegmentation.segments array
     * @param end index to the WindowSegmentation.segments array 
     * @param segList list of initialized Segments. 
     */    
    public Region(int regionNumber, int start, int end) {
	segmentA = start;
	segmentB = end;
	System.out.println("adding region: " + start + "," + end);
	drand = new DRand(new java.util.Date());
	setLayout(new BorderLayout());
        EmptyBorder eb = new EmptyBorder(5,5,5,5);
        SoftBevelBorder sbb = new SoftBevelBorder(SoftBevelBorder.LOWERED);
        setBorder(new EmptyBorder(5,5,5,5));

	this.regionNumber = regionNumber;
	// will house tempo, waveform display, and segment adjustment
	JPanel p2 = new JPanel();
        p2.setBorder(sbb);
        p2.setLayout(new BoxLayout(p2, BoxLayout.Y_AXIS));	
	add(p2, BorderLayout.CENTER);
	
        p2.setBorder(sbb);
	p2.setBorder(sbb);
        p2.setLayout(new BoxLayout(p2, BoxLayout.Y_AXIS));

	// add otherRegionsPanel
	otherRegionsPanel.setLayout(new BoxLayout(otherRegionsPanel,
						  BoxLayout.Y_AXIS));
	add(otherRegionsPanel, BorderLayout.EAST);
	add(makeWeightPanel());
	
	initFrame(regionNumber);
    }
}

/* 
 * how about I play a region at a time, 
 * so that I might be able to do different stuff later. 
 */
