package sound;

import javax.swing.*;
import java.awt.event.*;
import java.util.Vector;
import java.awt.Color;

import wavelet.WindowedSegmentation;
import  cern.jet.random.engine.DRand;


/** 
 * collection of regions
 */
public class RegionList extends JPanel implements ActionListener, Runnable
{
    int maxRegions = 10;
    int numRegions = 0;
    JButton [] regionBs;
   
    Vector regions = new Vector();
    Segment []    segment;
    int separationA, separationB;
    SamplingGraph sg;
    int currentSelectedRegion = 0;
    /**
     * indicates whether Segments have been calculated
     */
    boolean initialized = false;
    Thread thread;
    ByteChannel bc;

    JButton addRegionB, deleteRegionB;
    TopButtons topB;
    private boolean done = true;
    boolean justPlayOne = false;
    DRand rand;

    /** 
     * returns frame number of start of currently selected region. 
     **/
    public int getCurrentlySelectedRegionStart() {
	Region current = (Region)regions.get(currentSelectedRegion);
	System.out.println("current.segmentA: " + current.segmentB);
	return ((Integer)sg.separate.segments.get(current.segmentA)).intValue();
    }

    public int getCurrentlySelectedRegionEnd() {
	Region current = (Region)regions.get(currentSelectedRegion);
	System.out.println("current.segmentB: " + current.segmentB);
	return  ((Integer)sg.separate.segments.get(current.segmentB)).intValue();
    }

    public void setSg(SamplingGraph sg1)  {
	sg = sg1;
    }

    public void turnOnRegionButtons() {
	for (int i = 0; i < 10; i ++) {
	    regionBs[i].setEnabled(true);
	}
    }

    public void stopSound() {
	System.out.println("regionlist.stopSound");
	done = true;
	((Region)regions.get(currentSelectedRegion)).keepGoing = false;
	thread = null; // ????

	if (bc.outToFile) {
	    bc.endOfSoundFile();
	}
	bc.stopSound();
    }

    public void turnOffRegionButtons() {
	for (int i = 0; i < 10; i ++) {
	    regionBs[i].setEnabled(false);
	}
    }

    public void initialize(double noise, 
			   WindowedSegmentation winSeg, 
			   TopButtons top) {
	rand = new DRand(new java.util.Date()); 
	// initialize segments
	System.out.println("creating " + (winSeg.segments.size() -1) 
			   + " segments");
	segment = new Segment[winSeg.segments.size()-1];
	for (int i =0; i<winSeg.segments.size()-1; i++) {		
	    segment[i] = new Segment(i, winSeg);
	} 
	
	for (int i = 0; i< segment.length; i++) {
	    segment[i].setAmpDiffs(compareSegmentAmps(i));
	    segment[i].setLengthDiffs(compareSegmentLengths(i));
	}
	winSeg.calcAmplitudes();
	topB = top;
	initialized = true;
    }

    // perhaps I shouldn't take absolute value? 
    // might need this info if I want to manipulate timing. 
    private double [] compareSegmentLengths(int inx) {
	double [] lengths = new double[segment.length];
	for(int i =0; i < segment.length; i++ ) {
	    lengths[i] = Math.abs(segment[inx].length - segment[i].length);
	}
	return lengths;
    }
    
    private double [] compareSegmentAmps(int inx) {
	double [] amps = new double[segment.length];
	for(int i =0; i < segment.length; i++ ) {
	    amps[i] = Math.abs(segment[inx].avgAmplitude - 
			       segment[i].avgAmplitude);
	}
	return amps;
    }

    // these are now indexes to the WindowSegmentation.separations 
    // array!
    public void setRegionBoundary(int firstSeparation, int lastSeparation) {
	System.out.println("last region set: " + firstSeparation 
			   + "," + lastSeparation);
	separationA = firstSeparation;
	separationB = lastSeparation;
    }
    
    public void activateNewRegionButton() {
	addRegionB.setEnabled(true);
    }
    public void deactivateNewRegionButton() {
	addRegionB.setEnabled(false);
    }

    public void activateDeleteRegionButton() {
	deleteRegionB.setEnabled(true);
    }
    public void deactivateDeleteRegionButton() {
	deleteRegionB.setEnabled(false);
    }

    public RegionList(int maxregions) {
	maxRegions = maxregions;
	regionBs = new JButton[maxRegions];
	// create panel
	// regions boxes
	addRegionB = addButton("new region", this, false);
	deleteRegionB = addButton("delete region", this, false);
	for (int i = 0; i < maxRegions; i++ ) {
	    regionBs[i] = addButton(Integer.toString(i), this, false);
	}
    }

    public void add() {
	if (regions.size() < maxRegions - 1) {
	    System.out.println("adding region: " + separationA 
			       + "," + separationB);
	    Region newrl = new Region(regions.size(),separationA, separationB);
	    regions.add(newrl);
	    newrl.setOtherRegions(regions.size());

	    // add this region to all others. 
	    for (int i = 0; i < regions.size()-1; i++) {
		((Region)regions.get(i)).addOtherRegion();
	    }
	    activateDeleteRegionButton();	    
	    regionBs[regions.size()-1].setEnabled(true);
	}
	else {
	    System.out.println("sorry, already max number of regions");
	}	
    }

    public void delete() {
	
	System.out.println("deleting region: " + currentSelectedRegion);
	if (regions.size() > currentSelectedRegion){
	    regions.removeElementAt(currentSelectedRegion);
	}
	// remote this region to all others. 
	for (int i = 0; i < regions.size()-1; i++) {
	    ((Region)regions.get(i)).deleteOtherRegion(currentSelectedRegion);
	}
	regionBs[regions.size()].setEnabled(false);
	regionBs[regions.size()].setBackground(null);
    }

    private JButton addButton(String name, JPanel p, boolean state) {
        JButton b = new JButton(name);
        b.addActionListener(this);
        b.setEnabled(state);
        p.add(b);
        return b;
    }

    public void resetColors() {
	for (int i = 0; i < regions.size(); i++) {
	    regionBs[i].setBackground(null);
	}
    }

    public void showRegion(int regionNum, boolean showPanel) {
	currentSelectedRegion = regionNum; 
	resetColors();
	regionBs[regionNum].setBackground(Color.blue);
	Region r = (Region)regions.get(regionNum);
	// xx this might be trouble, as I think sg expects this
	// to be in X-coordinates. 
	sg.showRegion(r.segmentA, r.segmentB);
	if (showPanel) {
	    r.f.show();
	}
    }

    /** 
     * play some sound! 
     **/
    public void play(ByteChannel bch) {
	bc = bch;
	justPlayOne = false;
	done = false;

	if (regions.size() > 0) {	    
	    // make sure byteChannel available by now!
	    thread = new Thread(this);
	    thread.setName("Region");
	    thread.setDaemon(true);
	    thread.setPriority(9);
	    thread.start();
	} 
	else {	    
	    System.out.println("Please Define a region first");
	    topB.scrambleRegionB.setText("Scramble Region");
	    topB.playB.setEnabled(true);
	    topB.playRegionB.setEnabled(true);
	    topB.randSecB.setEnabled(true);
	}
    }

    public void playOne(ByteChannel bch) {
	bc = bch;
	System.out.println("playOne");
	justPlayOne = true;
	if (regions.size() > 0) {
	    // find currently selected region
	    // make sure byteChannel available by now!
	    thread = new Thread(this);
	    thread.setName("Region");
	    thread.start();
	} 
	else {
	    System.out.println("Please Define a region first");
	    topB.oneRegionB.setText(">");
	}
	topB.playB.setEnabled(true);
	topB.playRegionB.setEnabled(true);
	topB.randSecB.setEnabled(true);
    }

    public void run() {
	int nextRegion;
	Region currentRegion = (Region)regions.get(currentSelectedRegion);
	if (initialized == false) {
	    System.out.println("RegionList not initialized yet! ");
	}
	bc.segment = segment;
	currentRegion.play(bc, segment);	
	if (justPlayOne == false) {
	    while (!done) {
		nextRegion = currentRegion.nextRegion();
		currentRegion = (Region)regions.get(nextRegion);
		showRegion(nextRegion, false);
		currentRegion.play(bc, segment);
		System.out.println("Done!:  " + done); 
		// select this region
	    }
	}
	System.out.println("finished regionList thread");
	// after playing ends
	if (thread != null) {
	    topB.oneRegionB.setText(">");
	    topB.scrambleRegionB.setText("Scramble Region");
	    topB.playB.setEnabled(true);
	    topB.playRegionB.setEnabled(true);
	    topB.randSecB.setEnabled(true);
	    shutDown(null);
	}
    }
    // make Play Region, Scramble Region in Region List!
    private void shutDown(String message) {
	System.out.println("finished playing");
    }

    public void actionPerformed(ActionEvent e) {
	
        Object obj = e.getSource();
	if (obj.equals(addRegionB)) {
	    if (numRegions < maxRegions) {
		add();
	    }
	}
	else if (obj.equals(deleteRegionB)) {
	    delete();
	}
	else if (obj.equals(regionBs[0])) {
	    showRegion(0, true);
	}
	else if (obj.equals(regionBs[1])) {
	    showRegion(1, true);
	}
	else if (obj.equals(regionBs[2])) {
	    showRegion(2, true);
	}
	else if (obj.equals(regionBs[3])) {
	    showRegion(3, true);
	}
	else if (obj.equals(regionBs[4])) {
	    showRegion(4, true);
	}
	else if (obj.equals(regionBs[5])) {
	    showRegion(5, true);
	}
	else if (obj.equals(regionBs[6])) {
	    showRegion(6, true);
	}
	else if (obj.equals(regionBs[7])) {
	    showRegion(7, true);
	}
	else if (obj.equals(regionBs[8])) {
	    showRegion(8, true);
	}
	else if (obj.equals(regionBs[9])) {
	    showRegion(9, true);
	}
    }
}
