package sound;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;
import javax.swing.border.Border;
import javax.swing.BorderFactory; 
import  javax.swing.event.ChangeEvent;
import  javax.swing.event.ChangeListener;
import java.net.URL;

import utilities.DKnob;

public class SampleControls extends JPanel implements ActionListener{
    private static final boolean    DEBUG =  false;

    CapturePlayback capturePlayback;
    JButton segB  = new JButton("Segment");
    JButton playB = new JButton("Play");
    JCheckBox randWalkB = new JCheckBox("Walk");
    

    DKnob gainStart, gainMid, gainEnd;
    DKnob panStart, panMid, panEnd;
    JTextField trigField, durField;
    JTextField randTrig, randDur;
    ConfigParams configs;
    Border loweredbevel;
    Border etched;

    String soundFileName;

    public SampleControls() {
	configs = new ConfigParams();
	init();
    }

    public SampleControls(ConfigParams cpar) {
	configs = cpar;
	init();
    }

    public void resumeSettings() {
	capturePlayback = new CapturePlayback();

	if (configs.soundURL == null) {
	    capturePlayback.loadUp(configs.soundFileName, configs.segmented, 
				   configs.threshSlider,
				   configs.noiseSlider, configs.wavFilter,
				   configs.duration, configs.durVar,
				   configs.trigger, configs.trigVar,
				   configs.gainStart, configs.gainMid,
				   configs.gainEnd, configs.panStart, 
				   configs.panMid, configs.panEnd);
	}
	else  {
	    capturePlayback.loadURL(configs.soundURL, configs.segmented, 
				   configs.threshSlider,
				   configs.noiseSlider, configs.wavFilter,
				   configs.duration, configs.durVar,
				   configs.trigger, configs.trigVar,
				   configs.gainStart, configs.gainMid,
				   configs.gainEnd, configs.panStart, 
				   configs.panMid, configs.panEnd);
	}
	
	if (capturePlayback.audioSample != null) {
	    getSoundFileName();
	    segB.setText(soundFileName);
	}
	randWalkB.setSelected(configs.randWalk);
    }

    private void init() {
	loweredbevel = BorderFactory.createLoweredBevelBorder();
	Border redline = BorderFactory.createLineBorder(Color.blue);
	
        Border compound;
        compound = BorderFactory.createCompoundBorder(
                                  redline, loweredbevel);
	setBorder(compound);

	etched = BorderFactory.createEtchedBorder();
	
	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	setLayout(gridbag);
	c.fill = GridBagConstraints.HORIZONTAL; 
	
	JPanel gpPanel = new JPanel();	
	// two columns and as many rows as necessary. 
	gpPanel.setLayout(new GridLayout(0,2));
	///////////////////////////////////////////////////////
	// pan stuff
	JLabel panLabel = new JLabel("Pan", JLabel.CENTER);	
	panStart = new DKnob();
	panStart.addChangeListener(new ChangeListener()
	{
	    public void stateChanged(ChangeEvent ce)
	    {
		changePan();
	    }
	    });
	panStart.setVal(configs.panStart);

	panMid = new DKnob();
	panMid.addChangeListener(new ChangeListener()
	{
	    public void stateChanged(ChangeEvent ce)
	    {
		changePan();
	    }
	    });
	panMid.setVal(configs.panMid);

	panEnd = new DKnob();
	panEnd.addChangeListener(new ChangeListener()
	{
	    public void stateChanged(ChangeEvent ce)
	    {
		changePan();
	    }
	    });
	panEnd.setVal(configs.panEnd);
	JPanel panPanel = new JPanel();
	panPanel.setBorder(etched);	
	panPanel.setLayout(new BorderLayout());
	panPanel.add(panLabel, BorderLayout.NORTH);
	panPanel.add(panStart, BorderLayout.WEST);
	panPanel.add(panMid, BorderLayout.CENTER);
	panPanel.add(panEnd, BorderLayout.EAST);
	gpPanel.add(panPanel);

	///////////////////////////////////	
	// gain panel 
	JLabel gainLabel = new JLabel("Gain", JLabel.CENTER);	
	gainStart = new DKnob();    
	gainStart.addChangeListener(new ChangeListener()
	{
	    public void stateChanged(ChangeEvent ce)
	    {
		changeGain();
	    }
	    });
	gainStart.setVal(configs.gainStart);

	gainMid = new DKnob();    
	gainMid.addChangeListener(new ChangeListener()
	{
	    public void stateChanged(ChangeEvent ce)
	    {
		changeGain();
	    }
	    });
	gainMid.setVal(configs.gainMid);

	gainEnd = new DKnob();
	gainEnd.addChangeListener(new ChangeListener()
	{
	    public void stateChanged(ChangeEvent ce)
	    {
		changeGain();
	    }
	    });
	gainEnd.setVal(configs.gainEnd);

	JPanel gainPanel = new JPanel();
	gainPanel.setBorder(etched);	
	gainPanel.setLayout(new BorderLayout());
	gainPanel.add(gainLabel, BorderLayout.NORTH);
	gainPanel.add(gainStart, BorderLayout.WEST);
	gainPanel.add(gainMid, BorderLayout.CENTER);
	gainPanel.add(gainEnd, BorderLayout.EAST);
	gpPanel.add(gainPanel);
	add(gpPanel);

	/////////////////////////////////////////////////////
	// buttons
        segB.addActionListener(this);
	playB.addActionListener(this);
	
	CheckBoxListener myListener = new CheckBoxListener();
	randWalkB.addItemListener(myListener);
	

	JPanel buttonPanel = new JPanel();	
	buttonPanel.setLayout(new GridLayout(0,3));
	buttonPanel.add(segB);
	buttonPanel.add(playB); 
	randWalkB.setSelected(configs.randWalk);

	buttonPanel.add(randWalkB);
	

	//buttonPanel.add(segB);
	//buttonPanel.add(playB);
	
	c.gridy = 1;
	gridbag.setConstraints(buttonPanel, c);
	add(buttonPanel); 
	
	

	///////////////////////////////////

	JPanel durTrig = new JPanel();
	durTrig.setLayout(new GridLayout(0,2));

	JPanel durPan = new JPanel();
	JLabel durLabel = new JLabel("Duration");
	
	durField = new JTextField(String.valueOf(configs.duration));
	durField.setPreferredSize(new Dimension(40,25));
	durPan.add(durLabel);
	durPan.add(durField);

 	randDur = new JTextField(String.valueOf(configs.durVar));
	randDur.setPreferredSize(new Dimension(40,25));
	durPan.add(randDur);
	durTrig.add(durPan);

	JPanel trigPan = new JPanel();
	JLabel trigLabel = new JLabel("Trigger");
	trigField = new JTextField(String.valueOf(configs.trigger));
	trigField.setPreferredSize(new Dimension(40,25));
	trigPan.add(trigLabel);
	trigPan.add(trigField);
	randTrig = new JTextField(String.valueOf(configs.trigVar));
	randTrig.setPreferredSize(new Dimension(40,25));

	trigPan.add(randTrig);
	durTrig.add(trigPan);

	c.gridy = 2;
	gridbag.setConstraints(durTrig, c);
	add(durTrig);	
    }
    
    public void actionPerformed(ActionEvent e) {
        Object obj = e.getSource();
	if (obj.equals(segB)) {
	    if (capturePlayback == null) {
		capturePlayback = new CapturePlayback();
		capturePlayback.init();
		capturePlayback.f.show();
		if (DEBUG) {
		    System.out.println("load file");
		}
		capturePlayback.loadFile(null);
		if (capturePlayback.audioSample != null) {
		    getSoundFileName();
		    segB.setText(soundFileName);
		}
	    }
	    else {		
		capturePlayback.f.setVisible(true);
		/*
		  if (capturePlayback.file != null) {
			capturePlayback.createAudioSample(capturePlayback.file, 
		  true);
		}
		else {
		    capturePlayback.loadFile();
		}
		*/
		if (capturePlayback.audioSample != null) {
		    getSoundFileName();
		    segB.setText(soundFileName);
		}
	    }
	}
	else if (obj.equals(playB)) {
	    if (playB.getText().startsWith("Play")) {
		play();
	    }
	    else {
		stop();
	    }
	}
	
	else if ((capturePlayback != null) 
		 && (capturePlayback.byteChannel != null)) {
	    if ((obj.equals(trigField)) || (obj.equals(randTrig)))  {
		float trig = (new Float(trigField.getText().trim())).floatValue();
		float trigVar = (new Float(randTrig.getText().trim())).floatValue();
		capturePlayback.byteChannel.setPeriod(trig, trigVar);
	    }
	    else if ((obj.equals(durField)) || (obj.equals(randDur))) {
		float dur  = (new Float(durField.getText().trim())).floatValue();
		float durVar =  (new Float(randDur.getText().trim())).floatValue();
		capturePlayback.byteChannel.setChunkTime(dur, durVar);
	    }
	}
    }    

    private void getSoundFileName() {
	soundFileName = capturePlayback.audioSample.getFileName();
	int extStart = soundFileName.lastIndexOf("/")+ 1; 
	int extEnd = soundFileName.lastIndexOf("."); 

	if ((extEnd - extStart) > 10) {
	    extEnd  = extStart + 10;
	    if (DEBUG) {
		System.out.println("extEnd : " + extEnd + " extStart " 
				   + extStart);
	    }
	}
	soundFileName = soundFileName.substring(extStart,extEnd);
    }

    public void play() 
    {

	// doesn't handle case where sound is not scrambled. 

	if ((capturePlayback != null) && (capturePlayback.audioSample != null))
	    {
		setConfigParams();		    

		if (capturePlayback.separate.occupied) {
		    capturePlayback.randPlay(configs);
		}
		// regular play
		else {
		    capturePlayback.regPlay();
		}
		playB.setText("Stop");	    
	    }
    }

    public void stop() 
    {
	if (capturePlayback != null) {	    	    
	    if ((capturePlayback.byteChannel != null) 
		&& (capturePlayback.byteChannel.keepGoing)) {
		capturePlayback.byteChannel.stopSound();
	    }
	    // assume regular play. 
	    else {
		capturePlayback.playback.stop();
		capturePlayback.samplingGraph.stop();
	    }
	    playB.setText("Play");

	}
    }
    
    private void changeGain()
    {
	if (capturePlayback != null) {
	    if (capturePlayback.byteChannel != null) {		
		capturePlayback.byteChannel.setGain(gainStart.getValue(),
						    gainMid.getValue(), 
						    gainEnd.getValue());
	    }
	}
    }
   

    private void changePan()
    {
	float   fPanS = panStart.getValue();
	float   fPanM = panMid.getValue();
	float   fPanE = panEnd.getValue();
	
	if (DEBUG)
	{
	    System.out.println("AudioPlayerPanel.changeGain(): setting pan to "
			       + fPanS + " " + fPanM + " " + fPanE);
	}
	if (capturePlayback != null) {
	    if (capturePlayback.byteChannel != null) {
		capturePlayback.byteChannel.setPan(fPanS, fPanM, fPanE);
	    }
	}
    }
    
    public void setConfigParams() 
    {
	if ((capturePlayback != null) && (capturePlayback.audioSample != null))
	{
	    System.out.println("setting full configs " + 
			       capturePlayback.audioSample.getFileName());
	    System.out.println("gS: " + gainStart.getValue());
	    System.out.println("gM: " + gainMid.getValue());
	    System.out.println("gE: " + gainEnd.getValue());
	    System.out.println("pS: " + panStart.getValue()*0.01F);
	    System.out.println("pM: " + panMid.getValue()*0.01F);
	    System.out.println("pE: " + panEnd.getValue()*0.01F);
	    System.out.println("f : " + capturePlayback.audioSample.getFileName());
	    System.out.println("sep.occ: "
			       + capturePlayback.separate.occupied);
	    System.out.println("threshS: " + 
			       capturePlayback.threshSlider.getValue());
	    System.out.println("noiseS: " + 
			       capturePlayback.noiseSlider.getValue());
	    System.out.println("capturePlayback.wavFilter " + 
			       capturePlayback.wavFilter);
	    
	    configs.setParams(gainStart.getValue(), gainMid.getValue(),
			      gainEnd.getValue(), panStart.getValue(), 
			      panMid.getValue(), panEnd.getValue(),
			      capturePlayback.audioSample.getFileName(),
			      capturePlayback.separate.occupied,
			      capturePlayback.threshSlider.getValue(),
			      capturePlayback.noiseSlider.getValue(),
			      capturePlayback.wavFilter,
			      (new Float(trigField.getText().trim())).floatValue(),
			      (new Float(randTrig.getText().trim())).floatValue(),
			      (new Float(durField.getText().trim())).floatValue(),
			      (new Float(randDur.getText().trim())).floatValue(), 
			      randWalkB.isSelected());
	}
	else {
	    // System.out.println("setting only a few configs");
	    configs.setParams(
			      gainStart.getValue(),
			      gainMid.getValue(),
			      gainEnd.getValue(),
			      panStart.getValue(),
			      panMid.getValue(),
			      panEnd.getValue(),
			 (new Float(trigField.getText().trim())).floatValue(),
	       	         (new Float(randTrig.getText().trim())).floatValue(),
			 (new Float(durField.getText().trim())).floatValue(),
		         (new Float(randDur.getText().trim())).floatValue());
	}   
    }
	
    public static void main(String[] args) {
	
	SampleControls sc = new SampleControls();

	JFrame f = new JFrame("Segment Controls");
        f.getContentPane().add("Center", sc);
        f.pack();
        f.show();
    }

/** Listens to the check boxes. */
class CheckBoxListener implements ItemListener {
    public void itemStateChanged(ItemEvent e) {
	Object source = e.getItemSelectable();

	if (source == randWalkB) {
	    if (capturePlayback != null) {
		if (capturePlayback.byteChannel != null) {
		    capturePlayback.byteChannel.doWalk = true;
		    if (DEBUG) {
			System.out.println("rWalk: " + true);
		    }
		}
	    }
	}
	
	if (e.getStateChange() == ItemEvent.DESELECTED) {
	    if (capturePlayback != null) {
		if (capturePlayback.byteChannel != null) {
		    capturePlayback.byteChannel.doWalk = false;
		    if (DEBUG) {
			System.out.println("rWalk: " + false);	    
		    }
		}
	    }
	}
	
    }
}
    
}



