package sound;

import java.awt.*;
import java.awt.font.*;
import java.awt.event.*;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D.*;
import javax.swing.*;
import java.util.Vector;
import javax.sound.sampled.*;
import java.text.*;
import wavelet.WindowedSegmentation;

/**
 * Graphically render a WaveForm.
 */
class SamplingGraph extends JPanel implements Runnable, MouseListener,
					      MouseMotionListener
{
    private Thread thread;
    private Font font10 = new Font("serif", Font.PLAIN, 10);
    private Font font12 = new Font("serif", Font.PLAIN, 12);
    Color jfcBlue = new Color(204, 204, 255);
    Color pink = new Color(255, 175, 175);
	
    Vector  lines = new Vector();
    AudioSample audioSample;
    
    // Holds the coordinates of the user's last mousePressed event.
    // in the x coordinates of the Sampling graph. 
    public int     first_x, last_x;
    
    public boolean selecting = false;
    
    Vector  seps      = new Vector();
    public  String  errStr;
    static  boolean DEBUG = false;
    Playback playback;
    ByteChannel byteChannel;
    double seconds;
    RegionList regions;
	
    // records segmentNumbers that define Regions
    //  0 for first, 1 for end    
    int [] regionSegNumber = new int[2];
    
    WindowedSegmentation separate;
    
    public void setBounds(int x, int y, int width, int height)
    {
	super.setBounds(x,y,width,height);	    
	if (DEBUG) {
	   System.out.println("resized!");
	}
	if (audioSample != null) {
	    sketchWaveForm();
	    repaint();
	}
    } 
    
    public SamplingGraph(RegionList regionList) {
	setBackground(new Color(20, 20, 20));
	addMouseMotionListener(this);
	addMouseListener(this);
	regions = regionList;
	regions.setSg(this);
    }
	
    // Handles the event of the user pressing down the mouse button.
    public void mousePressed(MouseEvent e){
	first_x =  closeTo(e.getX(), false, 0);
	last_x =  first_x;
	regionSegNumber[1] = regionSegNumber[0];
	selecting = false;
	regions.deactivateNewRegionButton();
	//	    System.out.println("mouse pressed at " + last_x);
	repaint();
    }

    public void updateByteChannel(ByteChannel bc) {
	byteChannel = bc;
    }

    private int closeTo(int mouse, boolean more, 
			int segNumberIndex) {
	double nearestX  = 0;
	double nX;
	int i;
	for (i = 0; i < seps.size()-1; i++) {
	    if ((nX = ((Line2D.Double)(seps.get(i))).getX1()) > mouse) {
		if (more) {
		    regionSegNumber[segNumberIndex] = i;	
		    return (int)nX;
		} 
		else {
		    if (i>0) {
			regionSegNumber[segNumberIndex] = i-1;
		    }
		    else {
			regionSegNumber[segNumberIndex] = 0;
		    }
		    return (int)nearestX;
		}
	    }
	    else {
		nearestX = nX;
	    }
	}
	
	// must be last one. 
	// is the end defined as a separation?
	if (more) {
	    regionSegNumber[segNumberIndex] = i;			
	    return (int)(((Line2D.Double)(seps.lastElement())).getX1());
	}
	else {
	    if (i > 0) {
		regionSegNumber[segNumberIndex] = i-1;			
	    } 
	    else {
		regionSegNumber[segNumberIndex] = 0;			
	    }
	    return (int)nearestX;
	}
    }
    
    // Handles the event of a user dragging the mouse while holding
    // down the mouse button.
    public void mouseDragged(MouseEvent e){
	
	// xx if already segmented. 
	if (seps != null && seps.size() > 0) {
	    regions.activateNewRegionButton();
	    selecting = true;
	    last_x = closeTo(e.getX(), true,1);
	    repaint();
	}
    }    
    // Handles the event of a user releasing the mouse button.
    public void mouseReleased(MouseEvent e){
	if (DEBUG) {
	    System.out.println("mouse released at " +  e.getX() 
			       + "," + e.getY());
	}
	// send message to regionList
	// switch if reversed. 
	
	if (regionSegNumber[0] < regionSegNumber[1]) {
	    regions.setRegionBoundary(regionSegNumber[0], regionSegNumber[1]);
	}
	else {
	    regions.setRegionBoundary(regionSegNumber[1], regionSegNumber[0]);
	}
    }
    
    // This method is required by MouseListener.
    public void mouseMoved(MouseEvent e){}
    
    // These methods are required by MouseMotionListener.
    public void mouseClicked(MouseEvent e){}
    public void mouseExited(MouseEvent e){}
    public void mouseEntered(MouseEvent e){}
    
    public void reportStatus(String msg) {
	if ((errStr = msg) != null) {
	    System.out.println(errStr);
	    repaint();
	}
    }

    /** 
     * x, y are? 
     **/
   public void showRegion(int x, int y) {
	//	    System.out.println("mouse dragged at " +  e.getX() 
	//	       + "," + e.getY());
	Dimension d = getSize();
	int w = d.width;
	int frames_per_pixel = audioSample.signalCutoff/w;
	int numChannels = audioSample.format.getChannels();
	int sampleInc = (int)frames_per_pixel*numChannels;

	selecting = true;
	first_x = 
	    (((Integer)(separate.segments.get(x))).intValue() +2)*
	    separate.winIncrement/sampleInc;
	
	last_x = 
	    (((Integer)(separate.segments.get(y))).intValue() +2)*
	    separate.winIncrement/sampleInc;
	repaint();
    }
	
    private void sketchWaveForm() {
	seps.removeAllElements();
	lines.removeAllElements(); 
	
	Dimension d = getSize();
	int w = d.width;
	int h = d.height-15;
	if (DEBUG) {
	    System.out.println("w: " + w + " h " + h);
	}
	int [] audioData = audioSample.getDisplayFormat();
	audioSample.displayData = null;
	int frames_per_pixel = audioSample.signalCutoff/w;
	
	byte maxByte = 0;
	byte minByte = 0;
	
	double y_max = 0;
	double y_min = 0;
	int numChannels = audioSample.format.getChannels();
	int sampleInc = (int)frames_per_pixel*numChannels;
	if (DEBUG) {
	    System.out.println("SamplingGraph");
	    System.out.println("audio Data.length: " + audioData.length);
	    System.out.println("frames_per_pixel = " + frames_per_pixel);
	    System.out.println("sampleInc = " + sampleInc);
	}
	int idx;

	int maxAmpIndex =0;
	int maxAmp;
	int minAmpIndex = 0;
	int minAmp;
	
	for (int x = 0; x < w && audioData != null; x++) {
	    idx = sampleInc * x;
	    maxAmp = 0;
	    maxAmpIndex = idx;
	    minAmp = 0;
	    minAmpIndex = idx;
	    
	    // Take the max or min of the frames_per_pixel 
	    for (int i = idx; i < idx+sampleInc; i++) {
		// take max amplitude of frame, but keep sign
		if (audioData[i] > maxAmp) {
		    maxAmpIndex = i;
		    maxAmp = audioData[i];
		}
		else if (audioData[i] < minAmp) {
		    minAmpIndex = i;
		    minAmp = audioData[i];
		}
	    }
	    
	    if (audioSample.format.getSampleSizeInBits() == 8) {
		maxByte = (byte) audioData[maxAmpIndex];
		minByte = (byte) audioData[minAmpIndex];
	    } else {
		maxByte = (byte) (128 * audioData[maxAmpIndex] / 32768 );
		minByte = (byte) (128 * audioData[minAmpIndex] / 32768 );
	    }
	    y_max = (double) (h * (128 - maxByte) / 256);
	    y_min = (double) (h * (128 - minByte) / 256);
	    
	    lines.add(new Line2D.Double(x, y_max, x, y_min));
	}

	if (separate != null) {
	    double xCoord= 0;
	    
	    if (DEBUG) {
		System.out.println("organizing separations: " 
				   + separate.segments.size());
	    }
	    for (int i = 0; i < separate.segments.size(); i++)
	    { 
		xCoord = (double)
		    (((Integer)(separate.segments.get(i))).intValue() +2)*
		    separate.winIncrement/sampleInc;
		
		// System.out.println("adding sep: " + i + " Xcoord: " 
		// 		       + xCoord);
		seps.add(new Line2D.Double(xCoord, 0, xCoord,
					   h-1));
	    }
	}
	repaint();
    }
    
    public void createWaveForm(AudioSample audiosample, 
			       WindowedSegmentation separate, 
			       Playback playb) {
	// System.out.println("createwaveform");
	audioSample = audiosample;
	playback = playb; 
	this.separate = separate;
	    sketchWaveForm();	    
    }
    
    public void updateSeps(WindowedSegmentation separate) 
    {
	seps.removeAllElements();
	
	Dimension d = getSize();
	int w = d.width;
	int h = d.height-15;
	int frames_per_pixel = audioSample.signalCutoff/w;
	int numChannels = 1;
	int sampleInc = (int)frames_per_pixel*numChannels;
	
	if (separate != null) {
	    double xCoord= 0;
	    if (DEBUG) {
		System.out.println("organizing separations: " 
				   + separate.segments.size());
	    }
	    for (int i = 0; i < separate.segments.size(); i++){ 
		xCoord = (double)(((Integer)
				   (separate.segments.get(i))).intValue() 
				  +2)*separate.winIncrement/sampleInc;
		seps.add(new Line2D.Double(xCoord, 0, xCoord,
					   h-17));
	    }
	}
	repaint();
    }

    public void paint(Graphics g) {
	Dimension d = getSize();
	int w = d.width;
	int h = d.height;
	int INFOPAD = 15;
	    
	Graphics2D g2 = (Graphics2D) g;
	g2.setBackground(getBackground());
	g2.clearRect(0, 0, w, h);
	g2.setColor(Color.white);
	g2.fillRect(0, h-INFOPAD, w, INFOPAD);
	
	if (errStr != null) {
	    g2.setColor(jfcBlue);
	    g2.setFont(new Font("serif", Font.BOLD, 18));
	    g2.drawString("ERROR", 5, 20);
	    AttributedString as = new AttributedString(errStr);
	    as.addAttribute(TextAttribute.FONT,font12, 0, errStr.length());
	    AttributedCharacterIterator aci = as.getIterator();
	    FontRenderContext frc = g2.getFontRenderContext();
	    LineBreakMeasurer lbm = new LineBreakMeasurer(aci, frc);
	    float x = 5, y = 25;
	    lbm.setPosition(0);
	    while (lbm.getPosition() < errStr.length()) {
		TextLayout tl = lbm.nextLayout(w-x-5);
		if (!tl.isLeftToRight()) {
		    x = w - tl.getAdvance();
		}
		tl.draw(g2, x, y += tl.getAscent());
		y += tl.getDescent() + tl.getLeading();
	    }
	} 
	else if (audioSample != null) {
	    g2.setColor(Color.black);
	    g2.setFont(font12);
	    g2.drawString("File: " + audioSample.getFileName()
			  + "  Length: " +
			  String.valueOf(audioSample.audioBytes.length/
					 audioSample.getBytesPerSecond()) +
			  "  Position: " + String.valueOf(seconds),3, h-4);
	    if (selecting) {
		// draw selected rectangle
		g2.setPaint(Color.yellow);
		if (first_x < last_x) {
		    g2.fill(new Rectangle(first_x, 0,
					  last_x-first_x, 
					  h-INFOPAD-2));
		}
		else {
		    g2.fill(new Rectangle(last_x, 0,
					  first_x-last_x, 
					  h-INFOPAD-2));
		}
	    }
	    
	    // g2.setStroke(new BasicStroke(1.0f));
	    // .. render sampling graph ..                 
	    
	    g2.setColor(jfcBlue);
	    			    
	    for (int i = 1; i < lines.size(); i++) {
		g2.draw((Line2D) lines.get(i));
	    }
	    
	    // show separations, if any.
	    // System.out.println("Drawing separations: " + 
	    // 		    seps.size());
	    // g2.setStroke(new BasicStroke(2.0f));
	    g2.setColor(Color.red);
	    for (int i = 1; i < seps.size(); i++) {
		g2.draw((Line2D) seps.get(i));
	    }
	    
	    // .. draw current position ..
	    if (seconds != 0) {
		double loc = seconds/(audioSample.audioBytes.length/
				      audioSample.getBytesPerSecond())*w;
		g2.setColor(pink);
		g2.setStroke(new BasicStroke(3));
		g2.draw(new Line2D.Double(loc, 0, loc, h-INFOPAD-2));
	    }
            }
	regions.repaint();
    }
    
    public void start() {
	thread = new Thread(this);
	thread.setName("SamplingGraph");
	thread.start();
	seconds = 0;
    }

    public void stop() {
	if (thread != null) {
	    thread.interrupt();
	}
	thread = null;
    }
    
    public void run() {
	seconds = 0;
	while (thread != null) {
	    if ((playback.line != null) && (playback.line.isOpen()) ) {
		long milliseconds = 
		    (long)(playback.line.getMicrosecondPosition() / 1000);
		seconds =  milliseconds / 1000.0;
	    } 
	    else if ((byteChannel != null) && (byteChannel.keepGoing)) {
		long milliseconds = 
		    (long)(byteChannel.m_line.getMicrosecondPosition()
			       / 1000);
		seconds =  milliseconds / 1000.0;
	    }
	    
	    
	    try { thread.sleep(100); } catch (Exception e) { break; }
	    
	    repaint();
	    
	    while (playback.line != null && !playback.line.isOpen()) 
		{
		    try { thread.sleep(10); } 
		    catch (Exception e) { break; }
		}
	}
	seconds = 0;
	repaint();
    }
} // End class SamplingGraph
