package sound;

import javax.swing.*;        
import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;
import java.io.*;
import sound.*;
import java.net.URL;
import java.net.MalformedURLException;

public class Scrambler extends JPanel implements ActionListener
{
    JFrame f;  // top level 
    int numSampleControls = 6;
    SampleControls[] samples = new SampleControls[numSampleControls];
    File fn;
    JPanel p2;
    JPanel saveTFpanel;
    
    // load/save file
    JTextField textField;
    String fileName = "untitled";
    JButton savePresetB, loadPresetB;
    JButton playAll, stopAll;
    FileOutputStream objOut;
    static final boolean DEBUG = false;
    
    public Scrambler() {
	p2 = new JPanel();
	p2.setLayout(new GridLayout(0,2));
	init();
    }

    public void actionPerformed(ActionEvent e) {
        Object obj = e.getSource();
	if (obj.equals(savePresetB)) {
	    fileInOut(textField.getText().trim());
	    writeAllToFile();
	}
	else if (obj.equals(loadPresetB)) {
	    fileInOut(textField.getText().trim());
	    readFromFile();
	}
	else if (obj.equals(playAll)) {
	    for (int i = 0; i < numSampleControls; i++) {
		if (DEBUG) {
		    System.out.println("playing : " + i);
		}
		samples[i].play();
	    }	    
	}
	else if (obj.equals(stopAll)) {
	    for (int i = 0; i < numSampleControls; i++) {
		if (DEBUG) {
		    System.out.println("stopping : " + i);
		}
		samples[i].stop();
	    }
	}
    }

    public void init() {
	f = new JFrame("Audio Ecology Resynthesizer - Control Window");
        f.getContentPane().add("Center", this);
        f.pack();
	f.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
                    System.exit(0);
		}
	    });
	for (int i = 0; i < numSampleControls; i++) {	    
	    samples[i] = new SampleControls();
	}

        saveTFpanel = new JPanel();
	savePresetB = addButton("Save", saveTFpanel, true);
	loadPresetB = addButton("Load", saveTFpanel, true);
        saveTFpanel.add(new JLabel("File to load/save:  "));
        saveTFpanel.add(textField = new JTextField(fileName));
        textField.setPreferredSize(new Dimension(140,25));
        // getContentPane().add(saveTFpanel, BorderLayout.SOUTH);
        saveTFpanel.add(new JLabel("By Reynald Hoskinson"));
	playAll = addButton("play", saveTFpanel, true);
	stopAll = addButton("stop", saveTFpanel, true);
	redisplay();
    }

    public void fileInOut(String filename) 
    {
	try {
	    fn = new File(filename);
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }

    private JButton addButton(String name, JPanel p, boolean state) {
        JButton b = new JButton(name);
        b.addActionListener(this);
        b.setEnabled(state);
        p.add(b);
        return b;
    }
    
    // is this good enough?
    public URL checkIfURL(String t) {
	if (DEBUG) {
	    System.out.println("checking if " + t + " is a url");
	}
	URL temp = null;
	try {
	    temp = new URL(t);
	}
	catch (MalformedURLException e) {
	    System.out.println("soundfile not a url");
	}
	return temp;
    }
    
    public ConfigParams readComponent(BufferedReader bf) throws IOException
    {
	// read into ConfigParams object
	String buf;
	ConfigParams cparam = new ConfigParams();
	
	try {
	    buf = bf.readLine();
	    cparam.gainStart = (new Float(buf)).floatValue();
	    buf = bf.readLine();
	    cparam.gainMid = (new Float(buf)).floatValue();
	    buf = bf.readLine();
	    cparam.gainEnd = (new Float(buf)).floatValue();
	    buf = bf.readLine();
	    cparam.panStart = (new Float(buf)).floatValue();
	    buf = bf.readLine();
	    cparam.panMid = (new Float(buf)).floatValue();
	    buf = bf.readLine();
	    cparam.panEnd = (new Float(buf)).floatValue();

	    String filename = bf.readLine();
	    cparam.soundURL = checkIfURL(filename);
	    if (DEBUG) {
		System.out.println("URL: " + cparam.soundURL);
	    }
	    if (cparam.soundURL == null) {
		cparam.soundFileName = filename;
	    }    
	    buf = bf.readLine();
	    // System.out.println("bool buf: " + buf);
	    cparam.segmented = (buf.compareTo("false") != 0);
	    // cparam.segmented = Boolean.getBoolean(buf);
	    // System.out.println("bool bool: " + cparam.segmented);

	    buf = bf.readLine();
	    cparam.threshSlider = (Integer.decode(buf)).intValue();
	    buf = bf.readLine();
	    cparam.noiseSlider = (Integer.decode(buf)).intValue();
	    buf = bf.readLine();
	    cparam.wavFilter = (Integer.decode(buf)).intValue();
	    
	    buf = bf.readLine();
	    cparam.trigger = (new Float(buf)).floatValue();
	    buf = bf.readLine();
	    cparam.trigVar = (new Float(buf)).floatValue();


	    buf = bf.readLine();
	    cparam.duration = (new Float(buf)).floatValue();
	    buf = bf.readLine();
	    cparam.durVar = (new Float(buf)).floatValue();

	    buf = bf.readLine();
	    cparam.randWalk = (buf.compareTo("false") != 0);
	    // empty space. 
	    buf = bf.readLine();
	} 
	catch (NullPointerException e) {
	    throw new IOException();
	}
	return cparam;
    }

    private void redisplay() {
	p2.removeAll();
	removeAll();
	// reshow window. 
	for (int i = 0; i < numSampleControls; i++) {	    
	    p2.add(samples[i]);
	}
	add(p2,BorderLayout.CENTER);
        add(saveTFpanel, BorderLayout.SOUTH);
	doLayout();
	validate();
    }
    
    public void readFromFile() {
	try { 
	    Reader in = new FileReader(fn);
	    BufferedReader bf = new BufferedReader(in);
	    readPresets(bf);
	}
	catch (IOException e) {
	    System.out.println("Error: Preset file not readable");
	}
	redisplay();
    }

    public void readFromURL(URL url) {
	try { 
	    BufferedReader br = 
		new BufferedReader(new InputStreamReader(url.openStream()));
	    readPresets(br);
	}  
	catch (IOException e) {
	    System.out.println("Error: Preset file not readable");
	}
	redisplay();
    }
    
    public void readPresets(BufferedReader bf) throws IOException {
	ConfigParams cfp;
	for (int i = 0; i < numSampleControls; i++) {	 
	    if (DEBUG) {
		System.out.println("Reading component: " + i);
	    }
	    cfp = readComponent(bf);
	    samples[i] = new SampleControls(cfp);
	    samples[i].resumeSettings();
	}	    
    }

    public void writeAllToFile() {

	try {
	    String wf, outs;
	    int li;
	    Writer out = new FileWriter(fn);
	    PrintWriter pw = new PrintWriter(out);

	    for (int i = 0; i < numSampleControls; i++) {
		if (DEBUG) {
		    System.out.println("processing sample " + i);
		}
		samples[i].setConfigParams(); 
		samples[i].configs.printParams(pw);
		
		if ((samples[i].capturePlayback != null)  &&
		    (samples[i].capturePlayback.audioSample != null)) 
		{
		    // find wav filename root. 
		    wf = (samples[i].capturePlayback.audioSample.getFileName());
		    li = wf.lastIndexOf(".");
		    outs = wf.substring(0, li);
		    outs += ".obj";
		    objOut = new FileOutputStream(outs);
		    ObjectOutputStream objStream = 
			new ObjectOutputStream(objOut); 
		    objStream.writeObject(samples[i].capturePlayback.separate.transitions);
		    if (DEBUG) {
			System.out.println("saved object into " + outs);
		    }
		    objStream.flush();

		}
	    }
	pw.close();
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }
    
   public static void main(String[] args) {
	Scrambler si = new Scrambler();
	// process command line args. 
	if (args.length > 0) {
	    si.fileInOut(args[0]);
	    si.textField.setText(args[0]);
	    si.readFromFile();
	}
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

        si.setLocation(screenSize.width/5, screenSize.height/5);
	int w = 750;
	int h = 520;
	si.f.setSize(w, h);
	si.f.show();	
    }
}


