package sound;
import  wavelet.WindowedSegmentation;
import  utilities.Search;
import  cern.jet.random.engine.DRand;
import  cern.colt.Arrays;

import java.io.File;
import	java.io.FileOutputStream;
// import java.util.Arrays;
/**
 * determines which segment to play next: one for every segment. 
 */
public class Segment
{

    /**
     * start of this segment in frames
     */
    int startFrame;

    /**
     * end of this segment in frames
     */
    int endFrame;

    /** this is an array of how compatible each
     * segment is to the implicit one
     **/
    double[] m_MiddleTransitions;
    double[] m_EndTransitions;
    double[] m_StartTransitions;
    double[] m_JointTransitions;

    // re-normalized. 
    double[] m_normalizedTransitions;

    // original weighted
    double[] normalizedTransitions;

    /** 
     * average amplitude of this segment
     **/
    double avgAmplitude;
    double [] ampDiffs;

    /** 
     * number of samples in this segment
     **/
    double length;
    double [] lengthDiffs;
    /** 
     * index in the higher-level Segment Array
     */
    int m_Index;

    /**
     * cumulative density function
     **/
    double[] cdf;  

    /**
     *  noise parameter
     **/
    double   c; 
   
    DRand drand;
    WindowedSegmentation  separate;

    /**
     * penelty for repeating transitions. use from 2-10. 10 most penalty.  
     **/
    int keep = 10;
    
    int[] justPlayed = new int[keep];
    private static final boolean    DEBUG = false;
    private static final int    DEBUGLEVEL = -1;

    // file stuff 
    boolean                graph                = false; 
    FileOutputStream       outputStream;
    private File           file;
    private String []      outputString;
 

    
    /**
     * Returns reference to middleTransitions Array 
     */ 
    double [] getMiddleTransitions() {
	return m_MiddleTransitions;
    }

    /**
     * Returns reference to startTransitions Array 
     */ 
    double [] getStartTransitions() {
	return m_StartTransitions;
    }

    /**
     * Returns reference to endTransitions Array 
     */ 
    double [] getEndTransitions() {
	return m_EndTransitions;
    }

    /**
     * Returns reference to jointTransitions Array 
     */ 
    double [] getJointTransitions() {
	return m_JointTransitions;
    }
    
    public void setAmpDiffs(double [] ampd) {
	ampDiffs = ampd;
    }

    public void setLengthDiffs(double [] lengthD) {
	lengthDiffs = lengthD;
    }

    /** 
     * new way of generating stream
     * @param index : segment number
     **/
    public Segment(int index, WindowedSegmentation ws)
    {
	if (graph) {
	    try {
		file   = new File("normalize");
		outputStream = new FileOutputStream(file);
	    }
	    catch (Exception ex) { 
		ex.printStackTrace();
	    }
	}
	
	drand = new DRand(new java.util.Date()); 

	if (DEBUG) {
	    System.out.println("Constructing segment number: " + index);
	}
	m_Index = index;
	separate = ws;
	endFrame = ((Integer)separate.segments.get(index+1)).intValue();
	startFrame = ((Integer)separate.segments.get(index)).intValue(); 
	
	int totalNumberOfFrames =  ws.audioSample.signalCutoff / 
	    ws.audioSample.winIncrement;
	int totalLength = endFrame - startFrame; 
	//System.out.println("total number of frames: " + totalNumberOfFrames);
	//System.out.println("start: " + startFrame + " end: " + endFrame);
	length = totalLength/(double)totalNumberOfFrames;
	//System.out.println("length of segment: " + length);
	avgAmplitude = getAvgAmplitude() / (float)totalLength;
	// System.out.println("avg amplitude for this segment: "+ avgAmplitude);
	
	m_StartTransitions = ws.doCompatAtStart(index);
	m_MiddleTransitions = ws.doCompatAtMiddle(index);
	m_EndTransitions = ws.doCompatAtEnd(index);
	m_JointTransitions = ws.doCompatAtJoints(index);
	m_normalizedTransitions = new double[m_JointTransitions.length];
	normalizedTransitions = new double[m_JointTransitions.length];

	cdf = new double[m_StartTransitions.length]; 
    }

    private double getAvgAmplitude() {
	double ampValue = 0;
	for (int i = startFrame; i < endFrame; i++) {
	    ampValue += separate.transitions.getAvgEnergy(i);
	}
	return ampValue;
    }

    /**
     * computes the final weighting for each transition
     * the transitions range from 0 - 4
     * If I guarantee that all the weights are between 0 - 1
     * and divide the transitions by 4, everything will be 
     * between 0 - 1

     length, amplitude : must compare against rest !
     put comparison value here,instead of raw value!
     */
    public void computeTransitions(double noise,
				   float startWeight,
				   float middleWeight,
				   float endWeight, 
				   double [] jointProbs,
				   float jointWeight,
				   float lengthWeight,
				   float ampWeight) {
	c = noise;
	int denom = 0;
	if (startWeight != 0)
	    denom++;
	if (middleWeight != 0)
	    denom++;
	if (endWeight != 0)
	    denom++;
	if (jointWeight != 0)
	    denom++;
	if (lengthWeight != 0)
	    denom++;
	if (ampWeight != 0)
	    denom++;
	if (denom == 0) {
	    denom = 1;
	}
	/*
	if (DEBUGLEVEL == 2) {
	    System.out.println("ComputeTransitions: " + noise);
	    System.out.println("start: " + Arrays.toString(m_StartTransitions));
	    System.out.println("middle: " + Arrays.toString(m_MiddleTransitions));
	    System.out.println("end: " + Arrays.toString(m_EndTransitions));
	    System.out.println("joint: " + Arrays.toString(m_JointTransitions));
	}
	if (DEBUG) {
	    System.out.println("ComputeTransitions: " + noise);
	    System.out.println("start: " + startWeight);
	    System.out.println("middle: " + middleWeight);
	    System.out.println("end: " + endWeight);
	    System.out.println("joint: " + jointWeight);
	}
	*/
	// System.out.println("denom: "  + denom);
	if (jointWeight == 0) {
	    for (int i = 0; i < normalizedTransitions.length; i++) {
		normalizedTransitions[i] = 
		    m_StartTransitions[i]*startWeight + 
		    m_MiddleTransitions[i]*middleWeight + 
		    m_EndTransitions[i]*endWeight + 
		    lengthDiffs[i]*lengthWeight +
		    ampDiffs[i]*ampWeight; 
		normalizedTransitions[i] /= denom;
	    }
	}
	else if (jointProbs == null) {
	    for (int i = 0; i < normalizedTransitions.length; i++) {
		normalizedTransitions[i] = 
		    m_StartTransitions[i]*startWeight + 
		    m_MiddleTransitions[i]*middleWeight + 
		    m_EndTransitions[i]*endWeight + 
		    m_JointTransitions[i]*jointWeight + 
		    lengthDiffs[i]*lengthWeight +
		    ampDiffs[i]*ampWeight; 
		normalizedTransitions[i] /= denom;
	    }
	}
	// use provided joint probabilities
	else {
	    // System.out.println("use provided");
	    for (int i = 0; i < normalizedTransitions.length; i++) {
		normalizedTransitions[i] = 
		    m_StartTransitions[i]*startWeight + 
		    m_MiddleTransitions[i]*middleWeight + 
		    m_EndTransitions[i]*endWeight + 
		    jointProbs[i]*jointWeight + 
		    lengthDiffs[i]*lengthWeight +
		    ampDiffs[i]*ampWeight; 
		normalizedTransitions[i] /= denom;
	    }
	}
	normalize(normalizedTransitions);
    }

    /**
     * computes the final weighting for each transition
     * the transitions range from 0 - 4
     * If I guarantee that all the weights are between 0 - 1
     * and divide the transitions by 4, everything will be 
     * between 0 - 1

     length, amplitude : must compare against rest !
     put comparison value here,instead of raw value!
     */
    public void computeTransitions(double noise) {
	if (DEBUG) {
	    System.out.println("only using joint transitions");
	}
	c = noise;
	for (int i = 0; i < normalizedTransitions.length; i++) {
	    normalizedTransitions[i] =  m_JointTransitions[i];
	}
	normalize(normalizedTransitions);
    }


    // i'm changing the original translist here. normalizing it. 
    // is this a good thing?
    void normalize(double [] nt) 
    {
	int i;
	double   sumPi = 0.0;
	/*
	if (DEBUGLEVEL == 0) {
	    System.out.println("normalize: " + Arrays.toString(nt));
	}
	*/
	for( i =0; i< nt.length; i++ ) {
	    // need inverse since the lower the better.
	    // add one so we don't get a negative by 
	    // taking log of < 1. xs
	    m_normalizedTransitions[i] = 
		1/Math.log((1+nt[i])) + c;
	    sumPi += m_normalizedTransitions[i];
	}
	double cumu = 0;
	for( i =0; i< nt.length; i++ ) {
	    m_normalizedTransitions[i] = m_normalizedTransitions[i]/sumPi;
	    cumu += m_normalizedTransitions[i];
	    cdf[i] = cumu;
	}

	if ((graph) && (m_Index == 40)){
	    if (DEBUG) {
		System.out.println("index: " + m_Index);
	    }
	    String bl = new String();
	    // sort and print to a file for analyzation
	    try {
		double [] cop = new double[m_normalizedTransitions.length];
		System.arraycopy(m_normalizedTransitions, 0, cop, 0, cop.length);
		java.util.Arrays.sort(cop);
		
		for (int j = 0; j < cop.length; j ++) {
		    bl = cop[j] + "\n";
		    outputStream.write(bl.getBytes());
		}
		outputStream.close();
	    }
	    catch (Exception ex) { 
		ex.printStackTrace();
	    }		    
	}


	/*
	  if (DEBUG) {
	  System.out.println("cumu: " + cumu);
	  }
	*/
    }
    
    void insertPlayed(int segPlayed) {
	int i;
	for (i = 2; i<keep-1; i++) {
	    justPlayed[i] = justPlayed[i+1];
	}
	justPlayed[keep-1] = segPlayed;	
	// System.out.println("prob list: " + Arrays.toString(justPlayed));
    }

    int checkPlayed(int seg) {
	for (int i = 2; i< keep; i++ ) {
	    if (justPlayed[i] == seg) {
		return i;
	    }
	}
	return 1;
    }
    
    void reNormalize(int segPlayed) {
	int i = 0;
	int penalty;
	double   sumPi = 0.0;

	/*
	  if (DEBUG) {
	  System.out.println("current probability: " + 
	  m_normalizedTransitions[segPlayed]);
	}
	*/
	insertPlayed(segPlayed);
	sumPi = 0;
	for( i =0; i< normalizedTransitions.length; i++ ) {
	    // need inverse since the lower the better.
	    m_normalizedTransitions[i] = 1/Math.log((1+normalizedTransitions[i])) + c;
	    //   System.out.println("after inverse: " + m_normalizedTransitions[i]);
	    m_normalizedTransitions[i] /= checkPlayed(i);
	    sumPi  += m_normalizedTransitions[i];
	}
	
	double cumu = 0;
	for( i =0; i< normalizedTransitions.length; i++ ) {
	    m_normalizedTransitions[i] = m_normalizedTransitions[i]/sumPi;
	    cumu += m_normalizedTransitions[i];
	    cdf[i] = cumu;
	}
	/*
	  if (DEBUG) {
	  System.out.println("new probability: " 
	  + m_normalizedTransitions[segPlayed]);
	  }
	*/
    }
    /** 
     * now this function renormalizes
     * to lessen the chances of this being
     * returned again right away. 
     **/
    public int getIndex() {
	// System.out.println("m_transitions.length: " + m_transitions.length);
	if (normalizedTransitions.length < 2) {
	    return 0;
	}
	int seg;
	double candidate = drand.nextDouble();
	seg = Search.binarySearch(cdf, cdf.length, candidate);
	// seg = Math.abs(java.util.Arrays.binarySearch(cdf, candidate));
	/*
	if (DEBUG) {
	    System.out.println("got: " + seg + " could have got: " 
			       + java.util.Arrays.binarySearch(cdf, candidate));
	}
	*/
	reNormalize(seg);
	return seg;	
    }

    /*
    public int justGetIndex() {
	double candidate = drand.nextDouble();
	return Search.binarySearch(cdf, cdf.length, candidate);
    }
    */

}
