package sound;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import sound.CapturePlayback;
import sound.RegionList;

public class TopButtons extends JPanel implements ActionListener
{
    CapturePlayback cp;
    RegionList rl;
    JButton playRegionB, scrambleRegionB;
    JButton playB, captB, pausB, loadB, separateB;  
    JButton randSecB; 
    JButton oneRegionB;

    public TopButtons(CapturePlayback cpi, 
		      RegionList rli)
    {
	rl = rli;
	cp = cpi;
        playB= addButton("Play", this, false);
	pausB = addButton("Pause", this, false);
        loadB = addButton("Load...", this, true);
	separateB = addButton("Separate", this, false);
        randSecB = addButton("Random Play", this, false);
	playRegionB= addButton("Play Region", this, false);
	scrambleRegionB= addButton("Scramble Region", this, false);
	oneRegionB= addButton(">", this, false);
    }
    
    /**
     * utility function to assist in adding a gui button
     */
    private JButton addButton(String name, JPanel p, boolean state) {
        JButton b = new JButton(name);
        b.addActionListener(this);
        b.setEnabled(state);
        p.add(b);
        return b;
    }

    public void configureAfterSegmentation() {
	randSecB.setEnabled(true);
	playRegionB.setEnabled(true);
	scrambleRegionB.setEnabled(true);
	oneRegionB.setEnabled(true);
    }

    /**
     * action listener for buttons, etc. 
     */
    public void actionPerformed(ActionEvent e) {
	// System.out.println("Wavelet: " + wavFilter);
	
        Object obj = e.getSource();
        if (obj.equals(playB)) {
            if (playB.getText().startsWith("Play")) {
		pausB.setEnabled(true);
		playB.setText("Stop");
		cp.regPlay();
            } else {		
		pausB.setEnabled(false);
		playB.setText("Play");
		cp.stopSound();
	    }
	}
	else if (obj.equals(scrambleRegionB)) {
	    if (scrambleRegionB.getText().startsWith("Scramble")) {
		scrambleRegionB.setText("Stop");
		playB.setEnabled(false);
		randSecB.setEnabled(false);
		pausB.setEnabled(true);
		playRegionB.setEnabled(false);
		cp.playScrambledRegions();		
	    }
	    else { //stop
		scrambleRegionB.setText("Scramble Region");
		playB.setEnabled(true);
		randSecB.setEnabled(true);
		playRegionB.setEnabled(true);
		rl.stopSound();
	    }
	}
	else if (obj.equals(oneRegionB)) {
	    if (oneRegionB.getText().startsWith(">")) {
		oneRegionB.setText("stop");
		playB.setEnabled(false);
		randSecB.setEnabled(false);
		pausB.setEnabled(true);
		playRegionB.setEnabled(false);
		cp.playOneRegion();
	    }
	    else {
		oneRegionB.setText(">");
		playB.setEnabled(true);
		randSecB.setEnabled(true);
		playRegionB.setEnabled(true);
		rl.stopSound();
	    }
	} 
	else if (obj.equals(pausB)) {
            if (pausB.getText().startsWith("Pause")) {
		if (cp.playback.thread != null) {
		    cp.playback.line.stop();
		}
                pausB.setText("Resume");
            } else {
		if (cp.playback.thread != null) {
		    cp.playback.line.start();
		}
                pausB.setText("Pause");
            }
        } else if (obj.equals(loadB)) {
	    if (cp.loadFile(null)) {
		randSecB.setEnabled(false);
		playRegionB.setEnabled(false);
	    }
        }
	else if (obj.equals(separateB)) {
	    separateB.setEnabled(false);
	    cp.segmentSample(cp.threshSlider.getValue(), cp.wavFilter); 
	    separateB.setEnabled(true);
	}
	else if (obj.equals(playRegionB)) {
            if (playRegionB.getText().startsWith("Stop")) {
		cp.playClip.stop();
		cp.samplingGraph.stop();
		playRegionB.setText("Play Region");
	    }
	    else {
		pausB.setEnabled(true);
		playRegionB.setText("Stop");
		cp.playRegion();
	    }
	}
	else if (obj.equals(randSecB)) {	 			    
	    pausB.setEnabled(true);
	    playB.setText("Stop");	
	    cp.randPlay();	    
	}
    }
}
