/**
 * A class used to read Little Endian files in to Java.
 * Currently supports Int's, Short's and Strings only.
 *
 * @author Josh Richmond ( 2000/01/27 )
 */
package utilities;
import java.io.IOException;
import java.io.InputStream;

public class LittleEndianReader
{
    InputStream in;

    public LittleEndianReader(InputStream in)
    {
	this.in = in;
    }

    /**
     * Read 4 bytes, flip from Little to Big Endian, and return
     * the corresponding int.
     */
    public int readInt()
	throws IOException
    {
	byte buffer[] = new byte[4];
	
	in.read(buffer);

	int retVal = (((int) (buffer[3] & 0xff) << 24) |
		      ((int) (buffer[2] & 0xff) << 16) |
		      ((int) (buffer[1] & 0xff) <<  8) |
		      ((int) (buffer[0] & 0xff)));
	return(retVal);
    }

    /**
     * Read 2 bytes, flip from Little to Big Endian, and return
     * the corresponding short.
     */
    public short readShort()
	throws IOException
    {
	byte buffer[] = new byte[2];
	
	in.read(buffer);

	short retVal = (short) (((int) (buffer[1] & 0xff) << 8) |
				((int) (buffer[0] & 0xff)));
	return(retVal);
    }

    /**
     * Read n bytes, and store as a string.
     */
    public String readString(int nBytes)
	throws IOException
    {
	byte buffer[] = new byte[nBytes];
	in.read(buffer);

//  	int halfWay = (int) Math.floor((buffer.length-1) / 2.0);
//  	byte temp;

//  	for (int b = 0; b <= halfWay; b++)
//  	{
//  	    int x = buffer.length - b - 1;
//  	    temp = buffer[b];
//  	    buffer[b] = buffer[x];
//  	    buffer[x] = temp;
//  	}

	String retVal = new String(buffer);
	return(retVal);
    }

    /**
     * Read a byte array as-is. That is, no conversion is done.
     */
    public void readRawBytes(byte[] buffer)
	throws IOException
    {
	in.read(buffer);
    }

    /**
     * Skip n bytes.
     */
    public void skip(int n)
	throws IOException
    {
	in.skip(n);
    }
}
