
/**
 * A class used to write Little Endian files in to Java.
 * Currently supports Int's, Short's and Strings only.
 *
 * @author Josh Richmond ( 2000/01/27 )
 */

package utilities;
import java.io.IOException;
import java.io.OutputStream;

public class LittleEndianWriter
{
    OutputStream out;

    public LittleEndianWriter(OutputStream out)
    {
	this.out = out;
    }

    /**
     * Flip from Little to Big Endian, and write
     * the corresponding int.
     */
    public void writeInt(int val)
	throws IOException
    {
	byte buffer[] = new byte[4];
	
	for (int b = 0; b < 4; b++)
	    buffer[b] = (byte) ((val >> (b*8)) & 0xff);
	
	out.write(buffer);
    }

    /**
     * Flip from Little to Big Endian, and write
     * the corresponding short.
     */
    public void writeShort(short val)
	throws IOException
    {
	byte buffer[] = new byte[2];
	
	for (int b = 0; b < 2; b++)
	    buffer[b] = (byte) ((int) (val >> (b*8) & 0xff));

	out.write(buffer);
    }

    /**
     * Write the corresponding string.
     */
    public void writeString(String val)
	throws IOException
    {
	byte buffer[] = val.getBytes();
	
//  	int halfWay = (int) Math.floor((buffer.length-1) / 2.0);
//  	byte temp;

//  	for (int b = 0; b <= halfWay; b++)
//  	{
//  	    int x = buffer.length - b - 1;
//  	    temp = buffer[b];
//  	    buffer[b] = buffer[x];
//  	    buffer[x] = temp;
//  	}

	out.write(buffer);
    }

    /**
     * Write a byte array as-is. That is, no conversion is done.
     */
    public void writeRawBytes(byte[] buffer)
	throws IOException
    {
	out.write(buffer);
    }
}
