package utilities;

import	java.io.File;
import	java.io.IOException;
import	javax.sound.sampled.AudioInputStream;
import	javax.sound.sampled.AudioFormat;
import	javax.sound.sampled.AudioSystem;

import  javax.sound.sampled.SourceDataLine;
import  javax.sound.sampled.DataLine;
import  javax.sound.sampled.Mixer;

import  java.io.ByteArrayInputStream;
import  javax.sound.sampled.AudioFileFormat;

/** 
 * Misc Routines to facilitate sound file manipulation
 */

public class WavRoutines
{    
    private static final boolean    DEBUG = false;

    
    /** 
     * Gives power-of-two length portion of input signal given in 
     *  AudioInputStram audioIn.
     *  returns the power of two (depth) of the signal. 
    */
    public static double [] getPowerOfTwoDoubleData(AudioInputStream audioIn,
						    AudioFormat audioFormat)
    {
	int numBytes = 0;
	// converts sample in from bytes to doubles.
	// produces a power-of-two size sample
	try {
	    numBytes = audioIn.available();
	}
	catch (Exception e) {
	    e.printStackTrace();
	    System.exit(1);
	}
	
	int sizeInBytes = audioFormat.getSampleSizeInBits() / 8;
	int nSamples = numBytes / sizeInBytes;
	int depthT = (int)Math.floor((Math.log(nSamples)/Math.log(2)));  
	int inSignalCutoff = (int)Math.pow(2,depthT);
	
	// I could just read these in one at a time, instead of
	// all at once.  Integrate into loop below. 
	byte[] inBytes = new byte[inSignalCutoff*sizeInBytes];
	
	try {	    
	    audioIn.read(inBytes, 0, inSignalCutoff*sizeInBytes);
	}
	catch (Exception e) {
	    e.printStackTrace();
	    System.exit(1);
	}
	
	double[] dData = new double[inSignalCutoff];
	
	// 8-bit data
	if (sizeInBytes == 1) {
	    for (int s = 0; s < inSignalCutoff; s++) {
		// Since data is unsigned, make it signed
		short uVal = (short) (inBytes[s] & 0xff);
		dData[s] = (double) (uVal + Byte.MIN_VALUE) / 
		    (-Byte.MIN_VALUE);
	    }
	}
            
	// 16-bit data
	else if (sizeInBytes == 2) {
	    for (int s = 0; s < inSignalCutoff; s++) {
		short uVal = (short) (((int) (inBytes[s*2+1] & 0xff) << 8)
				      | ((int) (inBytes[s*2] & 0xff)));
		
		dData[s] = (double) ((double) uVal / (-Short.MIN_VALUE));
	    }
	}
	return dData;
    }    
    /** 
     * returns number of samples written to dData
     */
    public static int getDoubleData(byte[] inBytes, int inStart, int inEnd, 
				     double[] dData,
				     AudioFormat audioFormat)
    {
	int numBytes = inEnd - inStart ;
	int sizeInBytes = audioFormat.getSampleSizeInBits() / 8;
	int nSamples = numBytes / sizeInBytes;
	
	if (nSamples > dData.length) {
	    System.out.println("WavRoutines.getDoubleData ERROR: dData not big enough! (nSamples = " + nSamples + ", dData = " + dData.length);
	    return 0;
	}
	
	// 8-bit data
	if (sizeInBytes == 1) {
	    for (int s = 0; s < nSamples; s++) {
		// Since data is unsigned, make it signed
		short uVal = (short) (inBytes[s+inStart] & 0xff);
		dData[s] = (double) (uVal + Byte.MIN_VALUE) / 
		    (-Byte.MIN_VALUE);
	    }
	}
	// 16-bit data
	else if (sizeInBytes == 2) {
	    for (int s = 0; s < nSamples; s++) {
		short uVal = (short) (((int) (inBytes[(s*2+1) + inStart] 
					      & 0xff) << 8)
				      | ((int) (inBytes[(s*2) + inStart] 
						& 0xff)));
		dData[s] = (double) ((double) uVal / (-Short.MIN_VALUE));
	    }
	}
	return nSamples;
    }    
    
    public static double [] getDoubleData(byte[] inBytes,
					  AudioFormat audioFormat)
    {
	int numBytes = inBytes.length ;
	
	int sizeInBytes = audioFormat.getSampleSizeInBits() / 8;
	int nSamples = numBytes / sizeInBytes;
	
	// I could just read these in one at a time, instead of
	// all at once.  Integrate into loop below. 
	double[] dData = new double[nSamples];
	
	// 8-bit data
	if (sizeInBytes == 1) {
	    for (int s = 0; s < nSamples; s++) {
		// Since data is unsigned, make it signed
		short uVal = (short) (inBytes[s] & 0xff);
		dData[s] = (double) (uVal + Byte.MIN_VALUE) / 
		    (-Byte.MIN_VALUE);
	    }
	}
	
	// 16-bit data
	else if (sizeInBytes == 2) {
	    for (int s = 0; s < nSamples; s++) {
		short uVal = (short) (((int) (inBytes[s*2+1] & 0xff) << 8)
				      | ((int) (inBytes[s*2] & 0xff)));
		
		dData[s] = (double) ((double) uVal / (-Short.MIN_VALUE));
	    }
	}
	return dData;
    }    
    
    public static double [] getDoubleData(byte[] inBytes,
					  int numBytes,
					  AudioFormat audioFormat)
    {
	if (DEBUG) {
	    System.out.println("HERE 2.10"); 
	}    
	
	int sizeInBytes = audioFormat.getSampleSizeInBits() / 8;
	int nSamples = numBytes / sizeInBytes;
	
	// doesn't work for large samples!
	double[] dData = new double[nSamples];
	
	if (DEBUG) {
	    System.out.println("HERE 2.11"); 
	}    

	// 8-bit data
	if (sizeInBytes == 1) {
	    for (int s = 0; s < nSamples; s++) {
		// Since data is unsigned, make it signed
		short uVal = (short) (inBytes[s] & 0xff);
		dData[s] = (double) (uVal + Byte.MIN_VALUE) / 
		    (-Byte.MIN_VALUE);
	    }
	}
            
	if (DEBUG) {
	    System.out.println("HERE 2.12"); 
	}    

	// 16-bit data
	else if (sizeInBytes == 2) {
	    for (int s = 0; s < nSamples; s++) {
		short uVal = (short) (((int) (inBytes[s*2+1] & 0xff) << 8)
				      | ((int) (inBytes[s*2] & 0xff)));
		
		dData[s] = (double) ((double) uVal / (-Short.MIN_VALUE));
	    }
	}
	return dData;
    }    
		   


    /**
     * given an array of doubles(in this case, the inverse transform)
     * the signal representation in bytes is put into byteArray
     * the number of bytes put in is returned     
     */ 

    public static int getByteData(double[] dat, int datLength, 
				  byte[] byteArray, 
				  AudioFormat audioFormat) {
	/* support only 16-bit first */

	/* find out the length in bytes of the new sample */
	/* assume it has the same number of bits, etc. */

	int sizeInBytes = audioFormat.getSampleSizeInBits() / 8;
	int nSamples    = datLength * sizeInBytes;
	short tempShort;

	if (byteArray.length < nSamples) {
	    System.out.println("WavRoutines.getByteData ERROR: need to process " + nSamples + " but only have " + byteArray.length + " space ");
	}
	
	for (int s = 0; s < datLength; s++) {
	    tempShort = (short)(dat[s] * -Short.MIN_VALUE);
	    byteArray[s*2] = (byte) (tempShort & 0xff);	       
	    byteArray[s*2+1] = (byte) ((tempShort >> 8) & 0xff);
	}
	return nSamples;
    }


    static public int maxLengthInSamples(AudioInputStream audioInputStream, 
					 AudioFormat format) 
    {
	// System.out.println("MaxLength : " + maxLengthinSamples);

	return (((int)audioInputStream.getFrameLength()) 
		* (format.getFrameSize())*8)/format.getSampleSizeInBits();
    }

    public static AudioInputStream getAudioInputStream(String strFilename)
	{            
            File soundFile = new File(strFilename);           
            AudioInputStream audioInputStream = null;
 
            try {
                audioInputStream = 
                    AudioSystem.getAudioInputStream(soundFile);     
            }
            catch (Exception e) {
                e.printStackTrace();
                System.exit(1);
            }            
            return audioInputStream;    
        }


    public static void normalize(double [] inputSample) {
	
	double maxAmp = 0;
	int i;
	for (i=0; i< inputSample.length; i++) {
	    if (Math.abs(inputSample[i]) > maxAmp) 
		{
		    maxAmp = Math.abs(inputSample[i]);
		}
	}
	/*
	  so if maxAmp = .6, then we can normalize by 1/.6
	*/

	double scale = 1/maxAmp;

	for (i=0; i< inputSample.length; i++) {
	    inputSample[i] *= scale;
	}
    }

    // must it be governed by the max sample? 
    // could we do some sort of amplitude morphing? 
    // later, do compressor stuff. (when I learn how)

    // ratio: avgTo/avgFrom
    public static void normalizeTo(double [] inputSample, 
				   int inputSize, 
				   float ratio) {
	double maxAmp = 0;
	int i;

	if (ratio > 1.0) {
	    for (i=0; i< inputSize; i++) {
		if (Math.abs(inputSample[i]) > maxAmp) 
		    {
			maxAmp = Math.abs(inputSample[i]);
		    }
	    }
	
	    float scale = 1/(float)maxAmp;
	    if (scale < ratio) { 
		ratio= scale;
	    }
	}
	
	for (i=0; i< inputSize; i++) {
	    inputSample[i] *= ratio;
	}
    }
    
    public static void fileDoubles(double[] doubleStream, AudioFormat auf) 
    {

	int sizeInBytes = auf.getSampleSizeInBits() / 8;
	byte[] outBytes = new byte[sizeInBytes* doubleStream.length];
	getByteData(doubleStream, doubleStream.length, outBytes, auf);
	ByteArrayInputStream    bais = new ByteArrayInputStream(outBytes);
	AudioInputStream        ais = new AudioInputStream(bais, auf,
							   outBytes.length /
							   auf.getFrameSize());
	try
	    {
		AudioSystem.write(ais, AudioFileFormat.Type.WAVE, new File("out.wav"));
	    }
	catch (IOException e)
	    {
		e.printStackTrace();
	    }
    }


    public static void playDoubles(double[] doubleStream, AudioFormat auf) 
    {
	
	Mixer mixer    = AudioSystem.getMixer(null);	    

	// convert to bytes. 
	int sizeInBytes = auf.getSampleSizeInBits() / 8;
	byte[] outBytes = new byte[sizeInBytes* doubleStream.length];
	
	getByteData(doubleStream, doubleStream.length, outBytes, auf);

	SourceDataLine           m_line;

	try {
	    DataLine.Info   info = new DataLine.Info(SourceDataLine.class,auf);
	    m_line = (SourceDataLine) mixer.getLine(info);
	    m_line.open(auf, m_line.getBufferSize());
	    m_line.start();

	    int nBytesWritten = 
		m_line.write(outBytes, 0, outBytes.length);
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }
}


